options(show.signif.stars=F)
                                        #Correlation Rcmdr module- by Murray Logan 2004
correlation <- function(){
    if (!checkActiveDataSet()) return()
    if (!checkNumeric(2)) return()
    initializeDialog(title="Correlation")
    xBox <- variableListBox(top, Numeric(), selectmode="multiple", title="Variables (pick two or more)")
    radioButtons(name="correlations", buttons=c("pearson", "spearman", "kendall"), values=c("Pearson", "Spearman", "Kendall"),labels=c("Pearson product-moment", "Spearman rank-order", "Kendall's tau"), title="Type of Correlations")
    onOK <- function(){
        correlations <- tclvalue(correlationsVariable)
        x <- getSelection(xBox)
        varSelected <-x#xBox$varlist
        nSelected <- length(varSelected)
        if (2 > length(x)) {
            errorCondition(recall=correlationMatrix, message="Fewer than 2 variables selected.")
            return()
        }
        
        if (2==length(x)) {
          #if (.grab.focus) tkgrab.release(top)
          #      tkdestroy(top)
            if (correlations == "Pearson") doItAndPrint(paste("cor.test(",paste(x[1],x[2],sep=","),")",sep=""))
            if (correlations == "Spearman") doItAndPrint(paste("cor.test(",paste(x[1],x[2],sep=","),',method="spearman")',sep=""))
            if (correlations == "Kendall") doItAndPrint(paste("cor.test(",paste(x[1],x[2],sep=","),',method="kendall")',sep=""))
   
        }
        else {
          justDoIt(paste("corMat <-matrix(nrow=",nSelected,",ncol=",nSelected,")",sep=""))
          justDoIt(paste("dimnames(corMat) <-list(",list(x[1:nSelected]),",",list(x[1:nSelected]),")")) 
          justDoIt(paste("pMat <-matrix(nrow=",nSelected,",ncol=",nSelected,")",sep=""))
          justDoIt(paste("dimnames(pMat) <-list(",list(x[1:nSelected]),",",list(x[1:nSelected]),")")) 
          if (GrabFocus()) tkgrab.release(top)
          tkdestroy(top)
          x <- paste('"', x, '"', sep="")
           
          options(warn=-1)   
          if (correlations == "Pearson"){
            for(i in 1:nSelected){
              for (j in 1:nSelected) {
                justDoIt(paste("r <-cor.test(",paste(varSelected[i],varSelected[j],sep=","),")",sep=""))
                justDoIt(paste("corMat[",paste(i,j,sep=","),"] <-r$estimate",sep=""))
                justDoIt(paste("pMat[",paste(i,j,sep=","),"] <- r$p.value",sep=""))
              }
            }
            logger("# Pearson's correlation coefficient matrix")
            doItAndPrint(paste("corMat"))
            logger("# Matrix of associated probabilities")
            doItAndPrint(paste("pMat"))
       
          }
          else if (correlations == "Spearman"){
            #logger("# Spearman rank-order correlations")
            # doItAndPrint(paste("r<-cor(apply(", .activeDataSet, "[,c(", paste(x, collapse=","),')], 2, rank), use="complete.obs")', sep="")) 
            for(i in 1:nSelected){
              for (j in 1:nSelected) {
                justDoIt(paste("r <-cor.test(",paste(varSelected[i],varSelected[j],sep=","),', method="spearman")',sep=""))
                justDoIt(paste("corMat[",paste(i,j,sep=","),"] <-r$estimate",sep=""))
                justDoIt(paste("pMat[",paste(i,j,sep=","),"] <- r$p.value",sep=""))
              }
            }
            logger("# Spearman rank-order correlation coefficient matrix")
            doItAndPrint(paste("corMat"))
            logger("# Matrix of associated probabilities")
            doItAndPrint(paste("pMat"))
  
        }
        else if (correlations == "Kendall") {
            for(i in 1:nSelected){
              for (j in 1:nSelected) {
                justDoIt(paste("r <-cor.test(",paste(varSelected[i],varSelected[j],sep=","),', method="kendall")',sep=""))
                justDoIt(paste("corMat[",paste(i,j,sep=","),"] <-r$estimate",sep=""))
                justDoIt(paste("pMat[",paste(i,j,sep=","),"] <- r$p.value",sep=""))
              }
            }
          
            logger("# Kendall's correlation coefficient matrix")
            doItAndPrint(paste("corMat"))
            logger("# Matrix of associated probabilities")
            doItAndPrint(paste("pMat"))
  
        }
        
        }
        tkfocus(CommanderWindow())
    }                   
    options(warn=0)  
    OKCancelHelp(helpSubject="cor")
    tkgrid(getFrame(xBox), sticky="nw")
    tkgrid(correlationsFrame, sticky="w")
    tkgrid(buttonsFrame, sticky="w")
    dialogSuffix(rows=3, columns=1)
    }


#Basic Statistics Rcmdr module - by Murray Logan 2004
basicStatistics <- function(){
  if (!checkActiveDataSet()) return()
  if (!checkNumeric()) return()
  if (activeDataSet() == FALSE) {
        tkfocus(CommanderWindow())
        return()
        }
    if (length(Numeric()) == 0){
        tkmessageBox(message="There no numeric variables in the active data set.", 
                icon="error", type="ok")
        tkfocus(CommanderWindow())
        return()
        }

  initializeDialog(title="Basic Statistics")
  name <- tclVar("tab")
  nameFrame <- tkframe(top)
  nameEntry <- tkentry(nameFrame, width="20", textvariable=name)

  responseBox <- variableListBox(top, Numeric(), selectmode="multiple",title="Variable (pick one)")
  checkBoxes(frame="checkBoxFrame", boxes=c("mean", "median","var", "sd"), initialValues=c("1", "1","1","1"), labels=c("Mean", "Median","Variance","Standard deviation"))
  checkBoxes(frame="checkBoxFrame1", boxes=c("N","SEM", "CV"), initialValues=c("1","1", "1"), labels=c("Sample size","SEM", "CV"))
  quantilesVariable <- tclVar("1")
    quantilesFrame <- tkframe(top)
    quantilesCheckBox <- tkcheckbutton(quantilesFrame, variable=quantilesVariable)
    quantiles <- tclVar(".25,.75")
    quantilesEntry <- tkentry(quantilesFrame, width="20", textvariable=quantiles)

  ciVariable <- tclVar("1")
  ciFrame <- tkframe(top)
  ciCheckBox <- tkcheckbutton(ciFrame, variable=ciVariable)
  ci <- tclVar(".95")
  ciEntry <- tkentry(ciFrame, width="20", textvariable=ci)

  MeanVarPlotvar <- tclVar("0")
  MeanVarPlotFrame <- tkframe(top)
  MeanVarPlotCheckBox <-tkcheckbutton(MeanVarPlotFrame, variable=MeanVarPlotvar) 
  #otherVariable <- tclVar("1")
  #  otherFrame <-tkframe(top)
  #  otherCheckBox <- tkcheckbutton(otherFrame, variable=otherVariable)
  #  otherVariableText <- tclVar("")
  #  otherEntry <- tkentry(otherFrame, width="20", textvariable=otherVariableText)
  groupsBox(recall=basicStatistics, label="Summarize by:", initialLabel="Summarize by groups")
onOK <- function(){
        x <- getSelection(responseBox)
        tabName <- tclvalue(name)
        #v <- as.character(tkget(responseBox),"active")
        if (length(x) == 0){
            errorCondition(recall=numericalSummaries, message="You must select a variable.")
            return()
            }
        quants <- paste("c(", gsub(" ", ",", tclvalue(quantiles)), ")")
        cis <-   tclvalue(ci)
                   
        var <- paste(.activeDataSet, "$", x, sep="")
        if (GrabFocus()) tkgrab.release(top)
        tkdestroy(top)
        if (.groups == FALSE) {
          justDoIt(paste(tabName," <- rbind()"))
                
            if (tclvalue(meanVariable) == "1") {
              command <- paste("mean.1 <- sapply(list(",paste(var,collapse=","),"),mean)")
              justDoIt(command)
              justDoIt(paste(tabName,"<- rbind(",tabName,",Mean=round(mean.1,digits=3))",sep=""))
              #justDoIt(paste("tab <- rbind(tab,Mean=round(mean.1,digits=3))",sep=""))
              justDoIt(paste("rm(mean.1)"))
 
            }
            if (tclvalue(medianVariable) == "1") {
              #doItAndPrint(paste("median.1 <- median(", var, ", na.rm=TRUE)", sep=""))
              command <- paste("median.1 <- sapply(list(",paste(var,collapse=","),"),median)")
              justDoIt(command)
              justDoIt(paste(tabName, "<- rbind(",tabName,",Median=round(median.1,digits=3))",sep=""))
              justDoIt(paste("rm(median.1)"))
 
            }
            if (tclvalue(varVariable) == "1") {
              #doItAndPrint(paste("var.1 <-var(", var, ", na.rm=TRUE)", sep=""))
              command <- paste("var.1 <- sapply(list(",paste(var,collapse=","),"),var)")
              justDoIt(command)
              justDoIt(paste(tabName, "<- rbind(",tabName,",Variance=round(var.1,digits=3))",sep=""))
              justDoIt(paste("rm(var.1)"))
 
            }
            if (tclvalue(sdVariable) == "1") {
              #doItAndPrint(paste("sd.1 <-sd(", var, ", na.rm=TRUE)", sep=""))
              command <- paste("sd.1 <- sapply(list(",paste(var,collapse=","),"),sd)")
              justDoIt(command)
              justDoIt(paste(tabName, "<- rbind(",tabName,",SD=round(sd.1,digits=3))",sep=""))
              justDoIt(paste("rm(sd.1)"))
 
            }
            if (tclvalue(NVariable) == "1") {
              #doItAndPrint(paste("n.1 <-length(", var, ")",sep=""))
              command <- paste("n.1 <- sapply(list(",paste(var,collapse=","),"),length)")
              justDoIt(command)
              justDoIt(paste(tabName, "<- rbind(",tabName,",Size=round(n.1,digits=3))",sep=""))
              justDoIt(paste("rm(n.1)"))
 
            }
            if (tclvalue(SEMVariable) == "1") {
              #doItAndPrint(paste("se.1 <-sd(", var, ", na.rm=TRUE)/sqrt(length(", var, "))",sep=""))
              command <- paste("se.1 <- sapply(list(",paste(var,collapse=","),"),function(x) sqrt(var(x))/sqrt(length(x)))")
              justDoIt(command)
              justDoIt(paste(tabName, "<- rbind(",tabName,",SE=round(se.1,digits=3))",sep=""))
              justDoIt(paste("rm(se.1)"))
 
            }
            if (tclvalue(CVVariable) == "1") {
              command <- paste("var.1 <- sapply(list(",paste(var,collapse=","),"),var)")
              command <- paste("mean.1 <- sapply(list(",paste(var,collapse=","),"),mean)")
              justDoIt(paste("cv.1 <-var.1/mean.1"))
              justDoIt(paste(tabName, "<- rbind(",tabName,",CV=round(cv.1,digits=3))",sep=""))
              justDoIt(paste("rm(cv.1, mean.1, var.1)"))
 
            }
            if (tclvalue(ciVariable) == "1") {
              command <- paste("sd.1 <- sapply(list(",paste(var,collapse=","),"),sd)")
              justDoIt(command)
              command <- paste("mean.1 <- sapply(list(",paste(var,collapse=","),"),mean)")
              justDoIt(command)
              command <- paste("n.1 <- sapply(list(",paste(var,collapse=","),"),length)")
              justDoIt(command)
              command <- paste("cis <-",tclvalue(ci),sep="")
              justDoIt(command)
              justDoIt(paste("intval <- 1-((1-cis)/2)"))
              justDoIt(paste("ci.1 <-qnorm(intval,0,1)*(sd.1/sqrt(n.1))"))
              justDoIt(paste("ci.1 <- paste('+/-',round(ci.1,digits=3),sep='')"))
              justDoIt(paste(tabName, "<- rbind(",tabName,",CI=ci.1)",sep=""))
              justDoIt(paste("rm(ci.1,intval, cis, n.1,mean.1, sd.1)"))
 
            }
            if (tclvalue(quantilesVariable) == "1") {
              justDoIt(paste("q.1 <-quantile(", var, ", ",quants, ", na.rm=TRUE)", sep=""))
              justDoIt(paste(tabName, "<- rbind(",tabName,",QRange=q.1[2]-q.1[1])",sep=""))
              justDoIt(paste("rm(q.1)"))
 
            }
           command <-paste("colnames(",tabName,") <- c(",paste("'",x,"'",sep="",collapse=", "),")")
           #print(command)
           justDoIt(command)  
        }
        else {
          justDoIt(paste(tabName,"<- rbind()"))
            grps <- paste(.activeDataSet, "$", .groups, sep="")
            if (tclvalue(meanVariable) == "1") {
              #command<-paste("mean.1 <- tapply(sapply(list(",paste(var,collapse=","),"),length)", ", list(", grps,"), mean, na.rm=TRUE)", sep="")
              #print(command)
              justDoIt(paste("mean.1 <- tapply(", var, ", list(", grps,"), mean, na.rm=TRUE)", sep=""))
              justDoIt(paste(tabName,"<- rbind(",tabName,",Mean=round(mean.1,digits=3))",sep=""))
              justDoIt(paste("rm(mean.1)"))
 
            }
            if (tclvalue(medianVariable) == "1") {
              justDoIt(paste("median.1 <- tapply(", var, ", list(", grps,"), median, na.rm=TRUE)", sep=""))
              justDoIt(paste(tabName,"<- rbind(",tabName,",Median=round(median.1,digits=3))",sep=""))
              justDoIt(paste("rm(median.1)"))
 
            }
            if (tclvalue(varVariable) == "1") {
              justDoIt(paste("var.1 <- tapply(", var, ", list(", grps,"), var, na.rm=TRUE)", sep=""))
              justDoIt(paste(tabName,"<- rbind(",tabName,",Variance=round(var.1,digits=3))",sep=""))
              justDoIt(paste("rm(var.1)"))
 
            }
            if (tclvalue(sdVariable) == "1") {
              justDoIt(paste("sd.1 <- tapply(", var, ", list(", grps,"), sd, na.rm=TRUE)", sep=""))
              justDoIt(paste(tabName,"<- rbind(",tabName,",SD=round(sd.1,digits=3))",sep=""))
              justDoIt(paste("rm(sd.1)"))
 
            }
            if (tclvalue(NVariable) == "1") {
              justDoIt(paste("n.1 <- tapply(", var, ", list(", grps,"), length)", sep=""))
              justDoIt(paste(tabName,"<- rbind(",tabName,",SampleSize=round(n.1,digits=3))",sep=""))
              justDoIt(paste("rm(n.1)"))
 
            }
            if (tclvalue(SEMVariable) == "1") {
              justDoIt(paste("sd.1 <- tapply(", var, ", list(", grps,"), sd, na.rm=TRUE)", sep=""))
              justDoIt(paste("n.1 <- tapply(", var, ", list(", grps,"), length)", sep=""))
              justDoIt(paste("se.1 <- sd.1/sqrt(n.1)"))
              justDoIt(paste(tabName,"<- rbind(",tabName,",SEM=round(se.1,digits=3))",sep=""))
              justDoIt(paste("rm(se.1, sd.1, n.1)"))
 
            }
            if (tclvalue(CVVariable) == "1") {
              command <- paste("mean.1 <- tapply(", var, ", list(", grps,"), mean, na.rm=TRUE)", sep="")
              justDoIt(command)
              command <- paste("var.1 <- tapply(", var, ", list(", grps,"), var, na.rm=TRUE)", sep="")
              justDoIt(command)
              #justDoIt(command)
              justDoIt(paste("cv.1 <- var.1/mean.1"))
              justDoIt(paste(tabName,"<- rbind(",tabName,",CV=round(cv.1,digits=3))",sep=""))
              justDoIt(paste("rm(mean.1, var.1, cv.1)"))
 
            }
            if (tclvalue(ciVariable) == "1") {
              command <- paste("mean.1 <- tapply(", var, ", list(", grps,"), mean, na.rm=TRUE)", sep="")
              justDoIt(command)
              command <- paste("sd.1 <- tapply(", var, ", list(", grps,"), sd, na.rm=TRUE)", sep="")
              justDoIt(command)
              command <- paste("n.1 <- tapply(", var, ", list(", grps,"), length)", sep="")
              justDoIt(command)
              command <- paste("cis <-",tclvalue(ci),sep="")
              justDoIt(command)
              justDoIt(paste("intval <- 1-((1-cis)/2)"))
              justDoIt(paste("ci.1 <-qnorm(intval,0,1)*(sd.1/sqrt(n.1))"))
              justDoIt(paste("ci.1 <- paste('+/-',round(ci.1,digits=3),sep='')"))
              justDoIt(paste(tabName,"<- rbind(",tabName,",CI=ci.1)",sep=""))
              justDoIt(paste("rm(ci.1, intval, mean.1, sd.1, n.1, cis)"))
 
            }              
#            if (tclvalue(quantilesVariable) == "1") {
#              doItAndPrint(paste("q.1 <- tapply(", var, ", list(", grps,"), quantile, na.rm=TRUE, probs=", quants,")", sep=""))
#              justDoIt(paste("tab <- rbind(tab,Quantiles=q.1)",sep=""))
#            }
        }      
        tkfocus(CommanderWindow())
        #doItAndPrint(paste(tabName, "<-as.data.frame(t(",tabName,"))", sep=""))
        doItAndPrint(paste(tabName, "<-as.data.frame(",tabName,")", sep=""))

        if (tclvalue(MeanVarPlotvar) == "1"){
              justDoIt(paste("xvar <- as.numeric(t(",tabName,")[,'Mean'])"))
              justDoIt(paste("yvar <- as.numeric(t(",tabName,")[,'Variance'])"))
              doItAndPrint(paste("plot(xvar,yvar, xlab='Mean', ylab='Variance')", sep=""))
              justDoIt(paste("rm(xvar,yvar)"))
   
        }
        #clean up
                 }

  #map down the widgets
  OKCancelHelp()
  tkgrid(tklabel(nameFrame,text="Table name"),nameEntry,sticky="w")
  tkgrid(nameFrame,sticky="w")
  tkgrid(getFrame(responseBox), sticky="nw")
  tkgrid(checkBoxFrame,checkBoxFrame1, sticky="w")
  tkgrid(tklabel(quantilesFrame, text="Interquantiles"), quantilesCheckBox,
        tklabel(quantilesFrame, text=" quantile range:"), quantilesEntry, sticky="w")
    tkgrid(quantilesFrame, sticky="w")
  tkgrid(tklabel(ciFrame, text="Confidence Interval"), ciCheckBox,
        tklabel(ciFrame, text=" confidence:"), ciEntry, sticky="w")
    tkgrid(ciFrame, sticky="w")

  tkgrid(tklabel(MeanVarPlotFrame, text="Mean vs Var plot"), MeanVarPlotCheckBox ,sticky="w")
  tkgrid(MeanVarPlotFrame, sticky="w")
  #tkgrid(tklabel(otherFrame, text="Other"), otherCheckBox,
  #      tklabel(otherFrame, text=" function:"), otherEntry, sticky="w")
  #  tkgrid(otherFrame, sticky="w")
  tkgrid(groupsFrame, sticky="w")
  tkgrid(buttonsFrame, sticky="w")
  dialogSuffix(rows=6, columns=1)
   
 }

#Post Hoc test Rcmdr module - by Murray Logan 2004
postHocTest <- function(){
  require("multcomp")
 if (!checkActiveModel()) return()
  initializeDialog(title="Tukey's test")
  variableBox <- variableListBox(top, Factors(), title="Factor (pick one)")

  onOK <- function(){
      x <- getSelection(variableBox)
      #formul <- paste("strsplit(as.character(",.activeModel,'$call),"=")[2]')
      formul <- paste("formula(",.activeModel,'$call)',sep="")
      formul <- paste("formul<-formula(",.activeModel,'$call)',sep="")
doItAndPrint(formul)
      doItAndPrint(paste('attr(formul,".Environment") <-.GlobalEnv',sep=""))               
                     print(formul)
      command <- paste('summary(simtest(formul, type="Tukey", whichf="',x,'", data=',activeDataSet(),'))',sep="")
      #command <- paste("summary(simtest(formula(",.activeModel,'$call), type="Tukey", whichf="',x,'"))',sep="")
      #command <- paste("summary(simtest(",formula(.activeModel$call),', type="Tukey", whichf=',x,"))",sep="")
      doItAndPrint(command)               
      if (GrabFocus()) tkgrab.release(top)
        tkdestroy(top)
      remove(formul,envir=.GlobalEnv)
  }
  OKCancelHelp()
  tkgrid(getFrame(variableBox), sticky="nw")
  tkgrid(buttonsFrame, sticky="w")
  dialogSuffix(rows=3, columns=1)
  #doItAndPrint(paste("summary(simtest(",.activeModel,',type="Tukey"))',sep=""))
}


#bargraph function - by Murray Logan 2004
bargraph <- function(dv,iv,grp=NULL,
			width=1,space=.5,
			ymin,xmin=0,ymax,
			xlabs=c("N"),
			xmax,xlab,
			ylab1,ylab2,ylabs=c("N"),
			xlabelsfontsize=1,
			ylabelsfontsize=1,
			ytitlefontsize,
			ylabpos=4,
			sublabel=c(""),
			plotting=T,
			legnd,
			errorbars="None",
			...)
{
old.par <- par(no.readonly=T)  
on.exit(par(old.par))  
if (missing(ylab1)) ylab1 <- deparse(substitute(dv))
    if (missing(ylab2)) ylab2 <- ""
    #if (missing(xlab)) xlab <- deparse(substitute(iv))
    #space=.5
    if (missing(grp)) {
        means <-tapply(dv,iv,mean)
        stdev <- tapply(dv,iv,sd)
	n <- tapply(dv,iv,length)
	sterr <- stdev /sqrt(n)
	#xmax <- 0.5 + length(means)*(1 + 0.5)
	#barcolor <-c("grey")
	if (missing(xlab)) xlab <- deparse(substitute(iv))
	legnd<-NULL
    }
    else {
	#means <- tapply(dv,list(iv,grp),mean)
	#stdev <- tapply(dv,list(iv,grp),sd)
	#n <- tapply(dv,list(iv,grp),length)
        means <- tapply(dv,list(grp,iv),mean)
	stdev <- tapply(dv,list(grp,iv),sd)
	n <- tapply(dv,list(grp,iv),length)
	sterr <- stdev /sqrt(n)
        #xmax <- 15 #sum(c(length(means),rep(c(1,0),lev)))+3
#	 barcolor <-c(seq(0,length(levels(iv))-1))
	if (missing(xlab)) xlab <- deparse(substitute(iv))
	if (missing(legnd)) legnd <- c(levels(grp))
    }
    
    
    #if (missing(ymax)) {
    if(ymax=='NULL'){
       if (errorbars=="se") {
	   y.heights<-means+sterr + ((means+sterr)*.05)
       }
       else if (errorbars=="sd") {
	   y.heights=means+stdev+ ((means+stdev)*.05)
	   #print(y.heights)
       }
    else y.heights=means
    #ymax=ceiling(max(y.heights))
    ymax<-max(y.heights)
    #print(ymax)
    }

#print(ymin)
   if(ymin=='NULL'){
       ymin<-0
     }


    y <- seq(ymin,ymax,by=ymax/5)
    yy <- seq(ymin,ymax,by=ymax/25)
    
    if (ylabs=="N") ylabs <- seq(ymin,ymax,by=ymax/5)
    if (plotting==F) barcolor <-c("white")
    if (plotting==F) means <-c(0,0)#bordercolor <- c("white")
    par(lwd=2)
    #print(means)
    x <- barplot(means,
		#col=barcolor,
		ylab="",
		#space=space,
		axes=FALSE,
		ylim=c(ymin,ymax),
		#xlim=c(xmin,xmax),
		width=width,
		axisnames=FALSE,
		beside=T,
		xpd=F,
		legend.text=legnd,
		)
if (errorbars=="se") {
    if (plotting==T) arrows(x,means,x, means-sterr,angle=90,length=0.05,lwd=2.0)
    if (plotting==T) arrows(x,means,x, means+sterr,angle=90,length=0.05,lwd=2.0)
}
else if (errorbars=="sd") {
    if (plotting==T) arrows(x,means,x, means-stdev,angle=90,length=0.05,lwd=2.0)
    if (plotting==T) arrows(x,means,x, means+stdev,angle=90,length=0.05,lwd=2.0)
} 

#axis(2,las=1, pos=0.02, lwd=2,at=y,labels=ylabs,cex.axis=ylabelsfontsize,font=2)	
axis(2,las=1, lwd=2,cex.axis=ylabelsfontsize,font=2)	
#print(axTicks(2))
#segments(0,yy,0.02,yy)


if (missing(grp)) {
    x <- c(0.02,x)
    xlabs <- levels(iv)
}    
else {
    xlabs <- levels(iv)
    xleg <-max(x)
    xx <- split(x,gl(length(levels(iv)),length(levels(grp))))
    xx <- sapply(xx,mean)
    x <- c(0.02,xx)
    #print(xlabs)
}
print(xlabs)
xlabs<-c("",xlabs)

axis(1,at=x,pos=ymin,labels=xlabs,lwd=2,cex.axis=xlabelsfontsize,tick=TRUE,font=2)


mtext(ylab1,2,line=3,font=2,cex=1.5)
mtext(ylab2,2,line=2.2,font=2,cex=1.5)
mtext(xlab,1,line=3,font=2,cex=1.5)


#if(!missing(legnd)) legend(xleg-.5,ymax,legend=legnd) #legend(xleg + .5,ymax,legend=legnd,)
par(old.par)
x
}		


#Bargraph Rcmdr module (uses murray's bargraph function) - by Murray Logan 2004
Mbar <- function(){
  if (activeDataSet() == FALSE) {
        tkfocus(CommanderWindow())
        return()
        }
    if (length(Factors()) == 0){
        tkmessageBox(message="There are no factors in the active data set.", 
                icon="error", type="ok")
        tkfocus(CommanderWindow())
        return()
        }
    top <- tktoplevel()
    tkwm.title(top, "Bar Graph")

  #Frame 1 -
  frame1 <- tkframe(top,relief="groove",borderwidth=2,width=100)
  #tkgrid(tklabel(frame1,text="Variables",fg="blue"),sticky="nsew")
  #Dependent variable
  #dependentBox <- tklistbox(frame1, height=min(4, length(Numeric())),
  #      selectmode="single", background="white", exportselection="FALSE")
  dependentBox <- variableListBox(frame1, variableList=Numeric(), selectmode="single", title="Dependant variable (pick one)")
#  dependentScroll <- tkscrollbar(frame1, repeatinterval=5, 
#        command=function(...) tkyview(dependentBox, ...))
#  tkconfigure(dependentBox, yscrollcommand=function(...) tkset(dependentScroll, ...))
#    for (dep in Numeric()) tkinsert(dependentBox, "end", dep)
#  tkgrid(tklabel(frame1,text="Dependent"),dependentBox,dependentScroll,sticky="e")
tkgrid(getFrame(dependentBox), sticky="e")
  
  #Categorical variable
  variableBox <- variableListBox(frame1, variableList=Factors(), selectmode="single", title="Categorical variable (pick one)")
  tkgrid(getFrame(variableBox), sticky="e")
  #variableBox <- tklistbox(frame1, height=min(4, length(Numeric())),
  #      selectmode="single", background="white", exportselection="FALSE")
  #variableScroll <- tkscrollbar(frame1, repeatinterval=5, 
  #      command=function(...) tkyview(variableBox, ...))
  #tkconfigure(variableBox, yscrollcommand=function(...) tkset(variableScroll, ...))
  #  for (var in Factors()) tkinsert(variableBox, "end", var)
  #tkgrid(tklabel(frame1,text="Independent"),variableBox,variableScroll,sticky="e")

  #Grouping variable
  #groupingBox <- variableListBox(frame1, variableList=Factors(), selectmode="single", title="Categorical variable (pick one)")
  #tkgrid(getFrame(groupingBox), sticky="e")
  groupingBox <- tklistbox(frame1, height=min(4, length(Numeric())),
        selectmode="single", background="white", exportselection="FALSE")
  groupingScroll <- tkscrollbar(frame1, repeatinterval=5, 
        command=function(...) tkyview(groupingBox, ...))
  tkconfigure(groupingBox, yscrollcommand=function(...) tkset(groupingScroll, ...))
    for (grp in c("None",Factors())) tkinsert(groupingBox, "end", grp)
  tkgrid(tklabel(frame1,text="Grouping"),sticky="www")
 tkgrid(groupingBox,groupingScroll,sticky="ew")
  #options frame
  optionsFrame <- tkframe(top,relief="groove",borderwidth=2,width=100)
  tkgrid(tklabel(optionsFrame,text="Options",fg="blue"),sticky="nsew")
  errorbar <- tclVar("se")
   seRadiobox <- tkradiobutton(optionsFrame)
      tkconfigure(seRadiobox, variable=errorbar,value="se")
   sdRadiobox<- tkradiobutton(optionsFrame)
      tkconfigure(sdRadiobox, variable=errorbar,value="sd")
   noneRadiobox<- tkradiobutton(optionsFrame)
      tkconfigure(noneRadiobox, variable=errorbar,value="None")
  standarddeviation <- tclVar("0")
   sdCheckbox <-tkcheckbutton(optionsFrame, variable=standarddeviation)

  tkgrid(tklabel(optionsFrame, text="Include SE bars"), seRadiobox, sticky="w")
  tkgrid(tklabel(optionsFrame, text="Include STDEV bars"), sdRadiobox, sticky="w")
  tkgrid(tklabel(optionsFrame, text="Exclude error bars"), noneRadiobox, sticky="w")

  #axes labels and max/mins
  xLabText <- tclVar("")
  xLab <- tkentry(optionsFrame, width="20", textvariable=xLabText)
  tkgrid(tklabel(optionsFrame, text="x-axis label"), xLab, sticky="w")
yLabText <- tclVar("")
  yLab <- tkentry(optionsFrame, width="20", textvariable=yLabText)
  tkgrid(tklabel(optionsFrame, text="y-axis label"), yLab, sticky="w")
  ymin <- tclVar("")
  yMin <- tkentry(optionsFrame, width="20", textvariable=ymin)
  tkgrid(tklabel(optionsFrame, text="y minimum"), yMin, sticky="w")
  ymax <- tclVar("")
  yMax <- tkentry(optionsFrame, width="20", textvariable=ymax)
  tkgrid(tklabel(optionsFrame, text="y maximum"), yMax, sticky="w")

  symbols <-tclVar("")
  symbLab <-tkentry(optionsFrame, width="20", textvariable=symbols)
 
  tkgrid(tklabel(optionsFrame, text="symbols"), symbLab, sticky="w")
  
  #axes labels and max/mins
  xFrame <- tkframe(top)
  #xLabText <- tclVar("")
  #xLab <- tkentry(xFrame, width="20", textvariable=xLabText)
  #tkgrid(tklabel(xFrame, text="x-axis label"), xLab, sticky="w")

  #xmin <- tclVar("")
  #xMin <- tkentry(xFrame, width="20", textvariable=xmin)
  #tkgrid(tklabel(xFrame, text="x minimum"), xMin, sticky="w")
  #xmax <- tclVar("")
  #xMax <- tkentry(xFrame, width="20", textvariable=xmax)
  #tkgrid(tklabel(xFrame, text="x maximum"), xMax, sticky="w")
  
  yFrame <- tkframe(top)
  #yLabText <- tclVar("")
  #yLab <- tkentry(yFrame, width="20", textvariable=yLabText)
  #tkgrid(tklabel(yFrame, text="y-axis label"), yLab, sticky="w")
  #ymin <- tclVar("")
  #yMin <- tkentry(yFrame, width="20", textvariable=ymin)
  #tkgrid(tklabel(yFrame, text="y minimum"), yMin, sticky="w")
  #ymax <- tclVar("")
  #yMax <- tkentry(yFrame, width="20", textvariable=ymax)
  #tkgrid(tklabel(yFrame, text="y maximum"), yMax, sticky="w")
  

onOK <- function(){
        #depvariable <- as.character(tkget(dependentBox, "active"))
      depvariable <- getSelection(dependentBox)
        #variable <- as.character(tkget(variableBox, "active"))
      variable <- getSelection(variableBox)
        
        groupingvariable <- as.character(tkget(groupingBox, "active"))
      #groupingvariable <- getSelection(groupingBox)
   

        xlab <- as.character(tclvalue(xLabText))
        if(xlab=="") xl<-as.character(variable)
           else xl <- xlab
        ylab <- as.character(tclvalue(yLabText))
        if(ylab=="") yl<-as.character(depvariable)
           else yl <- ylab

        ym <- as.character(tclvalue(ymax))
        if(ym=="") ymx<-c("'NULL'")
           else ymx <- ym
        ymmn <- as.character(tclvalue(ymin))
        if(ymmn=="") ymn<-c("'NULL'")
           else ymn <- ymmn

        symbl <- as.character(tclvalue(symbols))
                   
        depvariable <- paste(.activeDataSet,"$",depvariable,sep="")
        variable <- paste(.activeDataSet,"$",variable,sep="")
        groupingvariable1 <- paste(.activeDataSet,"$",groupingvariable,sep="")
        
                  
        ErrorBar <- as.character(tclvalue(errorbar))
	
	if (ErrorBar=="None") err <-"\"None\""
	else if (ErrorBar == "se") err <- "\"se\""
  
	else err <- "\"sd\""
	
	if (GrabFocus()) tkgrab.release(top)
        tkdestroy(top)
        	
	if (groupingvariable == "None") command <-paste("b <- Mbargraph(", depvariable, ", ", variable, ",ymin=",ymn,",ymax=",ymx,",xlabel=",paste("'",xl,"'",sep=""),",ylabel=",paste("'",yl,"'",sep=""),",errorbars=", err, ", symbols=c(", symbl,"))")
	else command <-paste("b <- Mbargraph(", depvariable, ", list(", variable, ",", groupingvariable1,"),ymin=",ymn,",ymax=",ymx,",xlabel=",paste("'",xl,"'",sep=""),",ylabel=",paste("'",yl,"'",sep=""),", errorbars=", err,", symbols=c(", symbl,"))")
                   
	logger(command)
        justDoIt(command)
        
        tkfocus(CommanderWindow())
        }
    buttonsFrame <- tkframe(top)
    OKbutton <- tkbutton(buttonsFrame, text="OK", width="12", command=onOK, default="active")
    onCancel <- function() {
        if (GrabFocus()) tkgrab.release(top)
        tkfocus(CommanderWindow())
        tkdestroy(top)  
        }
    cancelButton <- tkbutton(buttonsFrame, text="Cancel", width="12", command=onCancel)
    onHelp <- function() {
        if (.Platform$OS.type != "windows") if (GrabFocus()) tkgrab.release(top)
        help(barplot)
        }
  OKCancelHelp()
  
  #draw out the grid
  tkgrid(frame1,row=0,column=0,sticky="news")
  tkgrid(optionsFrame,row=0,column=1,sticky="news")
  tkgrid(xFrame,yFrame,sticky="nw")
  tkgrid(buttonsFrame, row=2,column=0,sticky="w")
}

#Frequency distribution and Chisq test Rcmdr module - By Murray Logan 2004
goodnessofFit <- function(){
  #top <- tktoplevel()
  env <- environment()
  #if (!checkActiveDataSet()) return()
  #  if (!checkFactors()) return()
    initializeDialog(title="Frequency Distribution")
  if (!is.null(.activeDataSet)) if (length(Factors())>1) xBox <- variableListBox(top, Factors(), title="Variable (pick one)")# here #,command="FillTable()")
  #xBox <- tkListBox(top)#,command="FillTable()")
    
    #xBox <- variableListBox(top, Factors(), title="Variable (pick one)")#,command="FillTable()")
    #
    #tkgrid(getFrame(xBox),sticky="w")
    #tkbind(xBox$listbox, "<ButtonRelease-1>", FillTable)

   obsFrame <- tkframe(top)
   outerTableFrame <- tkframe(top)
   hypothesisFrame <- tkframe(top)
   #
   assign(".tableFrame", tkframe(outerTableFrame), envir=env)
   assign(".probsFrame", tkframe(hypothesisFrame), envir=env)
   #
    setUpTable <- function(...){
      
        tkdestroy(get(".tableFrame", envir=env))
        tkdestroy(get(".probsFrame", envir=env))

        assign(".tableFrame", tkframe(outerTableFrame), envir=env)
        assign(".probsFrame", tkframe(hypothesisFrame), envir=env)

        nrows <- as.numeric(tclvalue(rowsValue))
        nprows <- as.numeric(tclvalue(rowspValue))
        ncols <- as.numeric(tclvalue(colsValue))

        make.col.names <- "tklabel(.tableFrame, text='')"
        make.p.names <- "tklabel(.probsFrame, text='')"
        for (j in 1:ncols) {
            col.varname <- paste(".colname.", j, sep="")
            assign(col.varname, tclVar(j), envir=env)

            make.col.names <- paste(make.col.names, ", ", "tkentry(.tableFrame, width='10', textvariable=", 
                    col.varname, ")", sep="")
            make.p.names <- paste(make.p.names, ", ", "tkentry(.probsFrame, width='10', textvariable=", 
                    col.varname, ")", sep="")
                       
            }
        eval(parse(text=paste("tkgrid(", make.col.names, ")", sep="")), envir=env)
        eval(parse(text=paste("tkgrid(", make.p.names, ")", sep="")), envir=env)
        for (i in 1:nrows){   
            varname <- paste(".tab.", i, ".1", sep="") 
            varpname <-paste(".prob.", i, ".1", sep="") 
 
            assign(varname, tclVar("") , envir=env)
            assign(varpname, tclVar("") , envir=env)
                  
            row.varname <- paste(".rowname.", "1", sep="")
            row.p.varname <- paste(".rowpname.", "1", sep="")
            assign(row.varname, tclVar("obs"), envir=env)
            assign(row.p.varname, tclVar("exp"), envir=env)

            make.row <- paste("tkentry(.tableFrame, width='10', textvariable=",
                row.varname, ")", sep="")
            make.p.row <- paste("tkentry(.probsFrame, width='10', textvariable=",
                row.p.varname, ")", sep="")

            make.row <- paste(make.row, ", ", "tkentry(.tableFrame, width='10', textvariable=", 
                varname, ")", sep="")
            make.p.row <- paste(make.p.row, ", ", "tkentry(.probsFrame, width='10', textvariable=", 
                varpname, ")", sep="")
            for (j in 2:ncols){
                varname <- paste(".tab.", i, ".", j, sep="")
                varpname <-paste(".prob.", i, ".",j, sep="") 
 
                assign(varname, tclVar(""), envir=env)
                assign(varpname, tclVar(""), envir=env)

                make.row <- paste(make.row, ", ", "tkentry(.tableFrame, width='10', textvariable=", 
                    varname, ")", sep="")
                make.p.row <- paste(make.p.row, ", ", "tkentry(.probsFrame, width='10', textvariable=", 
                    varpname, ")", sep="")
                }
            eval(parse(text=paste("tkgrid(", make.row, ")", sep="")), envir=env)
            eval(parse(text=paste("tkgrid(", make.p.row, ")", sep="")), envir=env)
            }
        tkgrid(get(".tableFrame", envir=env), sticky="w")
        tkgrid(get(".probsFrame", envir=env), sticky="w")
        }

    FillTable <- function(){
      tkdestroy(get(".tableFrame", envir=env))
      tkdestroy(get(".probsFrame", envir=env))
      
      assign(".tableFrame", tkframe(outerTableFrame), envir=env)
      assign(".probsFrame", tkframe(hypothesisFrame), envir=env)
      x <- getSelection(xBox)
      command <- paste("table(", .activeDataSet, "$", x, ")", sep="")
      print(command)
      levs <- eval(parse(text=paste("levels(", .activeDataSet, "$", x, ")", sep="")))
      varTable <-eval(parse(text=paste("table(", .activeDataSet, "$", x, ")", sep="")))
      nrows <- 1
      n.levs <- length(levs)
      ncols <- n.levs
      make.col.names <- "tklabel(.tableFrame, text='')"
      make.p.names <- "tklabel(.probsFrame, text='')"
        for (j in 1:ncols) {
            col.varname <- paste(".colname.", j, sep="")
                           # print(col.varname)
            assign(col.varname, tclVar(levs[j]), envir=env)

            make.col.names <- paste(make.col.names, ", ", "tkentry(.tableFrame, width='10', textvariable=", 
                    col.varname, ")", sep="")
            make.p.names <- paste(make.p.names, ", ", "tkentry(.probsFrame, width='10', textvariable=", 
                    col.varname, ")", sep="")
                       
            }
        eval(parse(text=paste("tkgrid(", make.col.names, ")", sep="")), envir=env)
        eval(parse(text=paste("tkgrid(", make.p.names, ")", sep="")), envir=env)
      for (i in 1:nrows){   
            varname <- paste(".tab.", i, ".1", sep="") 
            varpname <-paste(".prob.", i, ".1", sep="") 
 
            assign(varname, tclVar(varTable[[i]]) , envir=env)
            assign(varpname, tclVar("") , envir=env)
                  
            row.varname <- paste(".rowname.", "1", sep="")
            row.p.varname <- paste(".rowpname.", "1", sep="")
            assign(row.varname, tclVar("obs"), envir=env)
            assign(row.p.varname, tclVar("exp"), envir=env)

            make.row <- paste("tkentry(.tableFrame, width='10', textvariable=",
                row.varname, ")", sep="")
            make.p.row <- paste("tkentry(.probsFrame, width='10', textvariable=",
                row.p.varname, ")", sep="")

            make.row <- paste(make.row, ", ", "tkentry(.tableFrame, width='10', textvariable=", 
                varname, ")", sep="")
            make.p.row <- paste(make.p.row, ", ", "tkentry(.probsFrame, width='10', textvariable=", 
                varpname, ")", sep="")
            for (j in 2:ncols){
                varname <- paste(".tab.", i, ".", j, sep="")
                varpname <-paste(".prob.", i, ".",j, sep="") 
 
                assign(varname, tclVar(varTable[j]), envir=env)
                assign(varpname, tclVar(""), envir=env)

                make.row <- paste(make.row, ", ", "tkentry(.tableFrame, width='10', textvariable=", 
                    varname, ")", sep="")
                make.p.row <- paste(make.p.row, ", ", "tkentry(.probsFrame, width='10', textvariable=", 
                    varpname, ")", sep="")
                }
            eval(parse(text=paste("tkgrid(", make.row, ")", sep="")), envir=env)
            eval(parse(text=paste("tkgrid(", make.p.row, ")", sep="")), envir=env)
            }
        tkgrid(get(".tableFrame", envir=env), sticky="w")
        tkgrid(get(".probsFrame", envir=env), sticky="w")
    }
    if (!is.null(.activeDataSet)) if (length(Factors())>1)tkgrid(getFrame(xBox),sticky="w")
  #here
    if (!is.null(.activeDataSet)) if (length(Factors())>1)tkbind(xBox$listbox, "<ButtonRelease-1>", FillTable)
   #here

    rowsValue <- tclVar("1")
    rowspValue <- tclVar("1")
    colsValue <- tclVar("2")
    colsSlider <- tkscale(obsFrame, from=2, to=10, showvalue=FALSE, variable=colsValue,
        resolution=1, orient="horizontal", command=setUpTable)
    colsShow <- tklabel(obsFrame, textvariable=colsValue, width=2, justify="right")
    

    


   onOK <- function(){
        nrows <- as.numeric(tclvalue(rowsValue))
        nprows <- as.numeric(tclvalue(rowspValue))
        ncols <- as.numeric(tclvalue(colsValue))
        cell <- 0
        counts <- rep(NA, nrows*ncols)
        counts.p <-rep(NA, nprows*ncols)
        row.names <- rep("", nrows)
        row.p.names <- rep("", nrows)
        col.names <- rep("", ncols)
        for (i in 1:nrows) {
          row.names[i] <- eval(parse(text=paste("tclvalue(", paste(".rowname.", i, sep=""),")", sep="")))
          row.p.names[i] <- eval(parse(text=paste("tclvalue(", paste(".rowpname.", i, sep=""),")", sep="")))
        }
        for (j in 1:ncols) col.names[j] <- 
            eval(parse(text=paste("tclvalue(", paste(".colname.", j, sep=""),")", sep="")))
        for (i in 1:nrows){
            for (j in 1:ncols){
                cell <- cell+1
                varname <- paste(".tab.", i, ".", j, sep="")
                varpname <- paste(".prob.", i, ".", j, sep="")
                counts[cell] <- as.numeric(eval(parse(text=paste("tclvalue(", varname,")", sep=""))))
                counts.p[cell] <- as.numeric(eval(parse(text=paste("tclvalue(", varpname,")", sep=""))))
                }
            }
        counts <- na.omit(counts)
        counts.p <- na.omit(counts.p)
        if (length(counts) != nrows*ncols){
            errorCondition(recall=enterTable, message=paste("Number of valid entries (", length(counts), ")\n",
                "not equal to number of rows (", nrows,") * number of columns (", ncols,").", sep=""))
            return()
            }
        if (length(counts.p) != nprows*ncols){
            errorCondition(recall=enterTable, message=paste("Number of valid expected values (", length(counts.p), ")\n",
                "not equal to number of rows (", nprows,") * number of columns (", ncols,").", sep=""))
            return()
            }
        if (length(unique(row.names)) != nrows){
            errorCondition(recall=enterTable, message="Row names are not unique.")
            return()
            }     
        if (length(unique(col.names)) != ncols){
            errorCondition(recall=enterTable, message="Column names are not unique.")
            return()
            }     
        if (GrabFocus()) tkgrab.release(top)
        tkdestroy(top)
         command <- paste("matrix(c(", paste(counts, collapse=","), "), ", nrows, ", ", ncols,
            ", byrow=TRUE)", sep="")
        counts.p<-counts.p/sum(counts.p)
        command.p <- paste("matrix(c(", paste(counts.p, collapse=","), "), ", nprows, ", ", ncols,
            ", byrow=TRUE)", sep="")
        assign(".Table", justDoIt(command), envir=.GlobalEnv)
        command <- paste("c(",paste(paste("'", row.names, "'", sep=""), collapse=", "), ")", sep="")
        justDoIt(paste("rownames(.Table) <- ", command, sep=""))
        command <- paste("c(",paste(paste("'", col.names, "'", sep=""), collapse=", "), ")", sep="")
        justDoIt(paste("colnames(.Table) <- ", command, sep=""))
        assign(".Probs", justDoIt(command.p), envir=.GlobalEnv)
        command <- paste("c(",paste(paste("'", row.p.names, "'", sep=""), collapse=", "), ")", sep="")
        justDoIt(paste("rownames(.Probs) <- ", command, sep=""))
        command <- paste("c(",paste(paste("'", col.names, "'", sep=""), collapse=", "), ")", sep="")
        justDoIt(paste("colnames(.Probs) <- ", command, sep=""))
        cnts <- sum(counts)*counts.p
        command.p <- paste("matrix(c(", paste(cnts, collapse=","), "), ", nprows, ", ", ncols,
            ", byrow=TRUE)", sep="")
        assign(".ExpCounts", justDoIt(command.p), envir=.GlobalEnv)
        command <- paste("c(",paste(paste("'", row.p.names, "'", sep=""), collapse=", "), ")", sep="")
        justDoIt(paste("rownames(.ExpCounts) <- ", command, sep=""))
        command <- paste("c(",paste(paste("'", col.names, "'", sep=""), collapse=", "), ")", sep="")
        justDoIt(paste("colnames(.ExpCounts) <- ", command, sep=""))
        
        justDoIt(paste("ChisqTable <- rbind(.Table,.ExpCounts)",sep=""))
        doItAndPrint("ChisqTable")
        doItAndPrint("chisq.test(.Table, p=.Probs)")
        remove(.Probs, envir=.GlobalEnv)
        remove(.Table, envir=.GlobalEnv)
        remove(.ExpCounts, envir=.GlobalEnv)
        
        tkfocus(CommanderWindow())
                    
     }
                    
  OKCancelHelp()
#tkgrid(getFrame(xBox),sticky="w")
  tkgrid(tklabel(obsFrame, text="Number of Columns:"), colsSlider, colsShow, sticky="w")
  tkgrid(obsFrame, sticky="w")
tkgrid(tklabel(top, text="Enter counts:", fg="blue"), sticky="w")
#tkgrid(outerTableFrame, getFrame(xBox),sticky="w")
  tkgrid(outerTableFrame,sticky="w")
  #tkbind(xBox, "<ButtonRelease-1>", FillTable)
tkgrid(tklabel(top, text="Enter expected ratio:", fg="blue"), sticky="w")
  tkgrid(hypothesisFrame, sticky="w")
  #tkgrid(getFrame(xBox), sticky="nw")    
    
    #tkgrid(optionsFrame, sticky="w")
    tkgrid(buttonsFrame, sticky="w")
    dialogSuffix(rows=3, columns=2)
  
}




enterTableM <- function(){
    env <- environment()
    initializeDialog(title="Enter and Analyze Two-Way Table")
    outerTableFrame <- tkframe(top)
    assign(".tableFrame", tkframe(outerTableFrame), envir=env)
    setUpTable <- function(...){
        tkdestroy(get(".tableFrame", envir=env))
        assign(".tableFrame", tkframe(outerTableFrame), envir=env)
        nrows <- as.numeric(tclvalue(rowsValue))
        ncols <- as.numeric(tclvalue(colsValue))
        make.col.names <- "tklabel(.tableFrame, text='')"
        for (j in 1:ncols) {
            col.varname <- paste(".colname.", j, sep="")
            assign(col.varname, tclVar(j), envir=env)
            make.col.names <- paste(make.col.names, ", ", "tkentry(.tableFrame, width='5', textvariable=", 
                    col.varname, ")", sep="")
            }
        eval(parse(text=paste("tkgrid(", make.col.names, ")", sep="")), envir=env)
        for (i in 1:nrows){   
            varname <- paste(".tab.", i, ".1", sep="") 
            assign(varname, tclVar("") , envir=env)
            row.varname <- paste(".rowname.", i, sep="")
            assign(row.varname, tclVar(i), envir=env)
            make.row <- paste("tkentry(.tableFrame, width='5', textvariable=",
                row.varname, ")", sep="")
            make.row <- paste(make.row, ", ", "tkentry(.tableFrame, width='5', textvariable=", 
                varname, ")", sep="")
            for (j in 2:ncols){
                varname <- paste(".tab.", i, ".", j, sep="")
                assign(varname, tclVar(""), envir=env)
                make.row <- paste(make.row, ", ", "tkentry(.tableFrame, width='5', textvariable=", 
                    varname, ")", sep="")
                }
            eval(parse(text=paste("tkgrid(", make.row, ")", sep="")), envir=env)
            }
        tkgrid(get(".tableFrame", envir=env), sticky="w")
        }
    rowColFrame <- tkframe(top)
    rowsValue <- tclVar("2")
    rowsSlider <- tkscale(rowColFrame, from=2, to=10, showvalue=FALSE, variable=rowsValue,
        resolution=1, orient="horizontal", command=setUpTable)
    rowsShow <- tklabel(rowColFrame, textvariable=rowsValue, width=2, justify="right")
    colsValue <- tclVar("2")
    colsSlider <- tkscale(rowColFrame, from=2, to=10, showvalue=FALSE, variable=colsValue,
        resolution=1, orient="horizontal", command=setUpTable)
    colsShow <- tklabel(rowColFrame, textvariable=colsValue, width=2, justify="right")
    onOK <- function(){
        nrows <- as.numeric(tclvalue(rowsValue))
        ncols <- as.numeric(tclvalue(colsValue))
        cell <- 0
        counts <- rep(NA, nrows*ncols)
        row.names <- rep("", nrows)
        col.names <- rep("", ncols)
        for (i in 1:nrows) row.names[i] <- 
            eval(parse(text=paste("tclvalue(", paste(".rowname.", i, sep=""),")", sep="")))
        for (j in 1:ncols) col.names[j] <- 
            eval(parse(text=paste("tclvalue(", paste(".colname.", j, sep=""),")", sep="")))
        for (i in 1:nrows){
            for (j in 1:ncols){
                cell <- cell+1
                varname <- paste(".tab.", i, ".", j, sep="")
                counts[cell] <- as.numeric(eval(parse(text=paste("tclvalue(", varname,")", sep=""))))
                }
            }
        counts <- na.omit(counts)
        if (length(counts) != nrows*ncols){
            errorCondition(recall=enterTableM, message=paste("Number of valid entries (", length(counts), ")\n",
                "not equal to number of rows (", nrows,") * number of columns (", ncols,").", sep=""))
            return()
            }
        if (length(unique(row.names)) != nrows){
            errorCondition(recall=enterTableM, message="Row names are not unique.")
            return()
            }     
        if (length(unique(col.names)) != ncols){
            errorCondition(recall=enterTableM, message="Column names are not unique.")
            return()
            }     
        percents <- as.character(tclvalue(percentsVariable))
        chisq <- tclvalue(chisqVariable)
        expected <- tclvalue(expFreqVariable)
        residuals <-tclvalue(residualsVariable)
               
        fisher <- tclvalue(fisherVariable)
        if (GrabFocus()) tkgrab.release(top)
        tkdestroy(top)
        command <- paste("matrix(c(", paste(counts, collapse=","), "), ", nrows, ", ", ncols,
            ", byrow=TRUE)", sep="")
        assign(".Table", justDoIt(command), envir=.GlobalEnv)
        logger(paste(".Table <- ", command, sep=""))
        command <- paste("c(",paste(paste("'", row.names, "'", sep=""), collapse=", "), ")", sep="")
        justDoIt(paste("rownames(.Table) <- ", command, sep=""))
        logger(paste("rownames(.Table) <- ", command, sep=""))
        command <- paste("c(",paste(paste("'", col.names, "'", sep=""), collapse=", "), ")", sep="")
        justDoIt(paste("colnames(.Table) <- ", command, sep=""))
        logger(paste("colnames(.Table) <- ", command, sep=""))
        doItAndPrint(".Table  # Counts")
        if (percents == "row") doItAndPrint("rowPercents(.Table) # Row Percentages")
        if (percents == "column") doItAndPrint("colPercents(.Table) # Column Percentages")
        if (chisq == 1) {
            command <- "chisq.test(.Table, correct=FALSE)"
            logger(paste(".Test <- ", command, sep=""))
            assign(".Test", justDoIt(command), envir=.GlobalEnv)
            doItAndPrint(".Test")
            if (expected == 1) doItAndPrint(".Test$expected # Expected Counts")
            warnText <- NULL
            if (0 < (nlt1 <- sum(.Test$expected < 1))) warnText <- paste(nlt1,
                "expected frequencies are less than 1")
            if (0 < (nlt5 <- sum(.Test$expected < 1))) warnText <- paste(warnText, "\n", nlt5,
                " expected frequencies are less than 5", sep="")
            if (!is.null(warnText)) tkmessageBox(message=warnText,
                icon="warning", type="ok")
            if (residuals == 1) doItAndPrint("residuals(.Test)")
            logger("remove(.Test)") 
            remove(.Test, envir=.GlobalEnv) 
            }
        if (fisher == 1) doItAndPrint("fisher.test(.Table)")
        logger("remove(.Table)") 
        remove(.Table, envir=.GlobalEnv)                                                      
        tkfocus(CommanderWindow())
        }
    OKCancelHelp(helpSubject="chisq.test")
    radioButtons(name="percents", buttons=c("rowPercents", "columnPercents", "nonePercents"), values=c("row", "column", "none"),
        initialValue="none", labels=c("Row percentages", "Column percentages", "No percentages"), title="Compute Percentages")
    checkBoxes(frame="testsFrame", boxes=c("chisq", "residuals","expFreq", "fisher"), initialValues=c("1", "1","0", "0"),
        labels=c("Chisquare test of independence", "Residuals","Print expected frequencies", "Fisher's exact test"))
    tkgrid(tklabel(rowColFrame, text="Number of Rows:"), rowsSlider, rowsShow, sticky="w")
    tkgrid(tklabel(rowColFrame, text="Number of Columns:"), colsSlider, colsShow, sticky="w")
    tkgrid(rowColFrame, sticky="w")
    tkgrid(tklabel(top, text="Enter counts:", fg="blue"), sticky="w")
    tkgrid(outerTableFrame, sticky="w")
    tkgrid(percentsFrame, sticky="w")
    tkgrid(tklabel(top, text="Hypothesis Tests", fg="blue"), sticky="w")
    tkgrid(testsFrame, sticky="w")
    tkgrid(buttonsFrame, columnspan=2, sticky="w")
    dialogSuffix(rows=7, columns=2)
    } 

princComp <- function(){
    if(!checkActiveDataSet()) return()
    if(!checkNumeric(2)) return()
    initializeDialog(title="Principal Components Analysis")
                       
    xBox <- variableListBox(top, Numeric(), selectmode="multiple", title="Variables (pick two or more)")
    subsetBox()
    checkBoxes(frame="optionsFrame", boxes=c("correlations", "screeplot", "ordination","addPC"), initialValues=c("1", "0", "0","0"),
        labels=c("Analyze correlation matrix", "Screeplot", "Ordination plot","Add principal components to data set"))
    onOK <- function(){
        x <- getSelection(xBox)
        nvar <- length(x)
        correlations <- tclvalue(correlationsVariable)
        subset <- tclvalue(subsetVariable)
        screeplot <- tclvalue(screeplotVariable)
        ordination <- tclvalue(ordinationVariable)
        addPC <- tclvalue(addPCVariable)
        if (2 > length(x)) {
            errorCondition(recall=princComp, message="Fewer than 2 variables selected.")
            return()
            }
        if (GrabFocus()) tkgrab.release(top)
        tkdestroy(top)
        subset <- if (trim.blanks(subset) == "<all valid cases>") "" else paste(", subset=", subset, sep="")
        correlations <- if (correlations == "1") "TRUE" else "FALSE"
        command <- paste("princomp(~", paste(x, collapse="+"), ", cor=", correlations,
            ", data=", .activeDataSet, subset, ")", sep="")
        assign(".PC", justDoIt(command), envir=.GlobalEnv)
        logger(paste(".PC <- ", command, sep=""))
        doItAndPrint("unclass(loadings(.PC))  # component loadings")
        doItAndPrint(".PC$sd^2  # component variances")
        opar <- par(ask=interactive())
        if (screeplot == "1") {
            justDoIt("screeplot(.PC)")
            logger("screeplot(.PC)")
            }
        if (ordination == "1") {
            justDoIt("biplot(.PC)")
            logger("biplot(.PC)")
            }               
        if (addPC == "1") {
            for(i in 1:nvar){
                var <- paste(".PC", i, sep="")
                if (is.element(var, .variables)) {
                    if ("no" == tclvalue(checkReplace(var))) next
                    }
                justDoIt(paste(.activeDataSet, "$.PC", i, " <- .PC$scores[,", i, "]", sep=""))
                logger(paste(.activeDataSet, "$.PC", i, " <- .PC$scores[,", i, "]", sep=""))
                }
            activeDataSet(.activeDataSet)
            remove(.PC, envir=.GlobalEnv)   
            }
        #doItAndPrint("PC <- .PC")               
        logger("remove(.PC)")
        tkfocus(CommanderWindow())
        }
    OKCancelHelp(helpSubject="princomp")
    tkgrid(getFrame(xBox), sticky="nw")
    tkgrid(subsetFrame, sticky="w")
    tkgrid(optionsFrame, sticky="w")
    tkgrid(buttonsFrame, sticky="w")
    dialogSuffix(rows=4, columns=1)
    }


mds <- function(){
    env <- environment()
#    if(!checkActiveDataSet()) return()
#    if(!checkNumeric(3)) return()
    if(!length(listDistances())) {
      tkmessageBox(message="There are no distances matricies.  Use the Distance measures procedure to generate a distance matrix, then get back to me!", 
                icon="error", type="ok")
        tkfocus(CommanderWindow())
        return()
    }
    initializeDialog(title="Multidimensional Scaling")
    command <-paste("names(",.activeDataSet,")")
    nameFrame <- tkframe(top)
    mName <- tclVar(paste(.activeDataSet,".mds", sep=""))
    mname <- tkentry(nameFrame, width="20", textvariable=mName)
    #xBox <- variableListBox(top, paste("names(".activeDataSet,")"), selectmode="multiple", title="Variables (pick three or more)")
    distBox <- variableListBox(top, variableList=listDistances(), selectmode="single", title="Distance matrix (pick one)")
    outerFrame <- tkframe(top)
#xBox <- variableListBox(top, justDoIt(command), selectmode="multiple", title="Samples (pick three or more)")
    #assign(".xB",xBox,envir=.GlobalEnv)
#    print(is.tkwin(get(".xB",envir=env)))        
    getDist <- function(){
      .activeDist <- getSelection(distBox)
      assign(".actDist",.activeDist,envir=.GlobalEnv)
      xBox <- variableListBox(top,attributes(justDoIt(.activeDist))$Labels, selectmode="multiple", title="Samples (pick three or more)")
      assign(".xB",xBox,envir=.GlobalEnv)
      tkgrid(getFrame(xBox),  sticky="w")
    }
                  
    #print("now:")
    #print(.xB)              
    
    #subsetBox()
    optionsFrame <- tkframe(top)
    nfactorVariable <- tclVar("2")
    nfactorEntry <- tkentry(optionsFrame, width="2", textvariable=nfactorVariable)
    checkFrame <- tkframe(top)
#
    checkBoxes(frame="checkBoxFrame", boxes=c("shepard", "configuration"),
        initialValues=c("0", "1"), labels=c("Shepard diagram", "Final configuration"))
  #title="Graphical Displays"
#
    onOK <- function(){
     # print("hello")
                       get(".xB",env=.GlobalEnv)
      
      x <- getSelection(.xB)#,envir=env)
      #print(x)
      #                 print("That")
        nvar <- length(x)
        nfactor <- as.numeric(tclvalue(nfactorVariable))
#
        shepard <- tclvalue(shepardVariable)
        configuration <- tclvalue(configurationVariable)
        if (3 > length(x)) {
            errorCondition(recall=mds, message="Fewer than 3 variables selected.")
            return()
            }
        f <- function(k, p) ((p - k)^2 - p - k)^2
        max.factors <- floor(optimize(f, c(0, nvar), tol=.0001, p=nvar)$minimum)
        if (nfactor > max.factors) {
            errorCondition(recall=mds, message=
                if (max.factors > 1) paste("Number of dimensions must be between 1 and ", max.factors, ".", sep="")
                    else "Number of dimensions cannot exceed 1.")
            return()
            }
        if (GrabFocus()) tkgrab.release(top)
        tkdestroy(top)
#       
        #command <- paste("isoMDS(as.dist(cbind(", paste(x, collapse=","), ")), k=", nfactor,")", sep="")
        #      assign(".m", justDoIt(command), envir=.GlobalEnv)
   #m <- as.matrix(dis)
   #mm <- m[2:6,2:6]
   #mmm <- as.dist(mm)
   #                    print(mmm)
        get(".actDist",env=.GlobalEnv)
        #doItAndPrint(.actDist)
        justDoIt(paste(".m <-as.matrix(",.actDist,")",sep=""))
        #doItAndPrint(x)
                       print(x)
        xx <- paste(paste("'",x,"'",sep=""),collapse=",")
                       print(xx)
        justDoIt(paste(".mm <- .m[c(",xx,"),c(",xx,")]",sep=""))
        justDoIt(paste(".mmm <- as.dist(.mm)"))              
        command <- paste("isoMDS(.mmm, k=",nfactor,")",sep="")
        assign(".mds", justDoIt(command), envir=.GlobalEnv)
        logger(paste(".mds <- ", command, sep=""))
                       
        justDoIt(".mds")
               
        #command <- paste("isoMDS(as.dist(cbind(", paste(x, collapse=","), ")), k=", nfactor,")", sep="")
        #      assign(".m", justDoIt(command), envir=.GlobalEnv)
        #logger(paste(".m <- ", command, sep=""))
                       
        #doItAndPrint(".m")
        #doItAndPrint('opar <- par(ask=interactive())')
        
if (shepard == 1){
          #command <- paste('.mds.shep <- Shepard(as.dist(cbind(', paste(x, collapse=","), ')),.m$points)')
          
	command <- paste('dev.set(1)')
          
	justDoIt(command)
          	
	command <- paste('.mds.shep <- Shepard(.mmm,.mds$points)')
          
	justDoIt(command)
#print(.mds.shep)
	command <- paste('plot(.mds.shep,pch="o",col="blue",xlab="MDS Distance",ylab="Dissimilarity Distance")')
          	doItAndPrint(command)
	command <- paste('lines(.mds.shep$x,.mds.shep$yf,type="s",col="red")')
          	
	doItAndPrint(command)
       	remove(.mds.shep, envir=.GlobalEnv)
  }
        
if (configuration == 1) {
          command <- paste('dev.set(1)')
          justDoIt(command)
          #justDoIt("dev.set(1)")
          command <- paste('plot(.mds$points,type="n",xlab="Dimension 1",ylab="Dimension 2")')
          doItAndPrint(command)
          command <- paste('text(.mds$points, labels =c(',paste('"',x,'"', collapse=","),'))')
         
doItAndPrint(command)
        }
        #doItAndPrint('par(opar)')               
#      
        doItAndPrint(paste(tclvalue(mName),' <- .mds'))
        logger("remove(.mds)")
        remove(.mds, envir=.GlobalEnv)
        remove(.m, envir=.GlobalEnv)
        remove(.mm, envir=.GlobalEnv)
        remove(.mmm, envir=.GlobalEnv)
        remove(.actDist, envir=.GlobalEnv)
        remove(.xB, envir=.GlobalEnv)
        ##remove(opar, envir=.GlobalEnv)
        tkfocus(CommanderWindow())
        }
    OKCancelHelp()
  tkgrid(getFrame(distBox), sticky="nw")
 tkbind(distBox$listbox, "<ButtonRelease-1>", getDist)
  tkgrid(tklabel(nameFrame, text="Enter name for model:"), mname, sticky="w")
  tkgrid(nameFrame, sticky="w")
#tkgrid(getFrame(xBox), sticky="nw")
  tkgrid(outerFrame, sticky="nw")
    tkgrid(tklabel(optionsFrame, text="Number of dimensions:"), nfactorEntry, sticky="w")
    tkgrid(optionsFrame, sticky="w")
 tkgrid(checkBoxFrame,sticky="w")
    tkgrid(checkFrame, sticky="w")
    tkgrid(buttonsFrame,  sticky="w")
    dialogSuffix(rows=5, columns=1)



              
    }



Distance <- function(){

 require("vegan")


if (!checkActiveDataSet()) return()
    if (!checkNumeric(2)) return()
    initializeDialog(title="Distance Measures")
    nameFrame <- tkframe(top)
    mName <- tclVar(paste(.activeDataSet,".dis", sep=""))
    mname <- tkentry(nameFrame, width="20", textvariable=mName)
    varsFrame <- tkframe(top)
    xBox <- variableListBox(varsFrame, Numeric(), selectmode="multiple", title="Variables (pick two or more)")
    command <- paste(".sites <- row.names(",.activeDataSet,")",sep="")
    justDoIt(command)
    siteBox<- variableListBox(varsFrame, .sites, selectmode="multiple", title="Samples/sites (pick two or more)")
    radioButtons(name="distances", buttons=c("bray", "euclidean", "canberra", "manhattan","kulczynski"), values=c("Bray-Curtis", "Euclidean", "Canberra", "Manhattan", "Kulczynski"),
        labels=c("Bray-Curtis", "Euclidean", "Canberra", "Manhattan", "Kulczynski"), title="Type of Distances")
    onOK <- function(){
        distances <- tclvalue(distancesVariable)
        x <- getSelection(xBox)
        s <- getSelection(siteBox)
             
        if (2 > length(x)) {
            errorCondition(recall=correlationMatrix, message="Fewer than 2 variables selected.")
            return()
            }
        if (2 > length(s)) {
            errorCondition(recall=correlationMatrix, message="Fewer than 2 sites selected.")
            return()
            }
        if ((distances == "partial") && (3 > length(x))) {
            errorCondition(recall=correlationMatrix, message="Fewer than 3 variables selected\nfor partial correlations.")
            return()
            }
        if (GrabFocus()) tkgrab.release(top)
        tkdestroy(top)
        x <- paste('"', x, '"', sep="")
        s <- paste('"', s, '"', sep="")
               
        if (distances == "Bray-Curtis")
            #doItAndPrint(paste(tclvalue(mName),"<- as.data.frame(as.matrix(vegdist(", .activeDataSet, "[,c(", paste(x, collapse=","),
            #    ')], method="bray")))', sep=""))
            doItAndPrint(paste(tclvalue(mName),"<- vegdist(", .activeDataSet, "[c(",paste(s, collapse=","),"),c(", paste(x, collapse=","),')], method="bray")', sep=""))
        else if (distances == "Euclidean")
            #doItAndPrint(paste(tclvalue(mName),"<- as.data.frame(as.matrix(vegdist(", .activeDataSet, "[,c(", paste(x, collapse=","),')], method="euclidean")))', sep=""))
                       doItAndPrint(paste(tclvalue(mName),"<- vegdist(", .activeDataSet, "[c(",paste(s, collapse=","),"),c(", paste(x, collapse=","),')], method="euclidean")', sep=""))
        else if (distances == "Canberra"){
            #doItAndPrint(paste(tclvalue(mName),"<- as.data.frame(as.matrix(vegdist(", .activeDataSet, "[,c(", paste(x, collapse=","),')], method="canberra")))', sep=""))
            doItAndPrint(paste(tclvalue(mName),"<- vegdist(", .activeDataSet, "[c(",paste(s, collapse=","),"),c(", paste(x, collapse=","),')], method="canberra")', sep=""))
             }
        else if (distances == "Manhattan"){
            #doItAndPrint(paste(tclvalue(mName),"<- as.data.frame(as.matrix(vegdist(", .activeDataSet, "[,c(", paste(x, collapse=","),')], method="manhattan")))', sep=""))
            doItAndPrint(paste(tclvalue(mName),"<- vegdist(", .activeDataSet, "[c(",paste(s, collapse=","),"),c(", paste(x, collapse=","),')], method="manhattan")', sep=""))
             }
        else doItAndPrint(paste(tclvalue(mName),"<- vegdist(", .activeDataSet, "[c(",paste(s, collapse=","),"),c(", paste(x, collapse=","),')], method="kulcyynski")', sep=""))
        tkfocus(CommanderWindow())
        }
    OKCancelHelp(helpSubject="vegdist")
  tkgrid(tklabel(nameFrame, text="Enter name for distance matrix:"), mname, sticky="w")
  tkgrid(nameFrame, sticky="w")
    tkgrid(varsFrame,sticky="nw")
    tkgrid(getFrame(xBox),getFrame(siteBox), sticky="nw")
tkgrid(varsFrame,sticky="nw")
    tkgrid(distancesFrame, sticky="w")
    tkgrid(buttonsFrame, sticky="w")
    dialogSuffix(rows=3, columns=1)
}


trim.blanks <- function(text){
    gsub("^\ ", "", gsub("\ *$", "", text))
    }


importSYSTAT <- function() {
    initializeDialog(title="Import SYSTAT Data Set")
    dsname <- tclVar("Dataset")
    entryDsname <- tkentry(top, width="20", textvariable=dsname)
    onOK <- function(){
        dsnameValue <- trim.blanks(tclvalue(dsname))
        if (dsnameValue == ""){
            errorCondition(recall=importSYSTAT,
                message="You must enter the name of a data set.")
                return()
                }     
        if (!is.valid.name(dsnameValue)){
            errorCondition(recall=importSYSTAT,
                message=paste('"', dsnameValue, '" is not a valid name.', sep=""))
            return()
            }
        if (is.element(dsnameValue, listDataSets())) {
            if ("no" == tclvalue(checkReplace(dsnameValue, "Data set"))){
                if (GrabFocus()) tkgrab.release(top)
                tkdestroy(top)
                importSYSTAT()
                return()
                }
            }
        file <- tclvalue(tkgetOpenFile(
            filetypes='{"SYSTAT data files" {".sys" ".syd" ".SYS" ".SYD"}} {"All Files" {"*"}}'))
        if (file == "") {
            if (GrabFocus()) tkgrab.release(top)
            tkdestroy(top)
            return()
            }
        command <- paste('read.systat("', file,'")', sep="")
        datalist <- justDoIt(command)
        lengths <- sapply(datalist, length)
        datalist <- datalist[lengths != 0]
        lengths <- lengths[lengths != 0]
        if (!all(lengths == length(datalist[[1]]))){
            tkmessageBox(message=
                paste("SYSTAT data set contains elements of unequal length.\nData set cannot be converted."),
                icon="error", type="ok")
            tkdestroy(top)
            tkfocus(CommanderWindow())
            return()
            }
        assign(dsnameValue, as.data.frame(datalist), envir=.GlobalEnv)
        logger(paste(dsnameValue, " <- as.data.frame(", command, ")", sep=""))
        activeDataSet(dsnameValue)
        if (GrabFocus()) tkgrab.release(top)
        tkdestroy(top)
        tkfocus(CommanderWindow())
        }
    OKCancelHelp(helpSubject="read.mtp")
    tkgrid(tklabel(top, text="Enter name for data set:"), entryDsname, sticky="e")
    tkgrid(buttonsFrame, columnspan="2", sticky="w")
    tkgrid.configure(entryDsname, sticky="w")
    dialogSuffix(rows=2, columns=2, focus=entryDsname)
    }


plannedComparisons <- function(){
    env <- environment()
    if (!checkActiveDataSet()) return()
    if (!checkActiveModel()) return()
    if (!checkFactors()) return()
    initializeDialog(title="Planned Comparisons")
    variableBox <- variableListBox(top, Factors(), title="Factor (pick one)")

    outerTableFrame <- tkframe(top)
    assign(".tableFrame", tkframe(outerTableFrame), envir=env)
    FillTable <- function(){
      tkdestroy(get(".tableFrame", envir=env))
      assign(".tableFrame", tkframe(outerTableFrame), envir=env)
      variable <- getSelection(variableBox)
      row.names <- eval(parse(text=paste("levels(", .activeDataSet, "$", variable, ")")))
      row.names <- substring(paste(abbreviate(row.names, 12), "            "), 1, 12)
      nrows <- length(row.names)
      ncols <- nrows - 1
      make.col.names <- "tklabel(.tableFrame, text='Contrast Name:')"
      for (j in 1:ncols) {
                varname <- paste(".col.", j, sep="")
                assign(varname, tclVar(paste(".", j, sep="")), envir=env)
                make.col.names <- paste(make.col.names, ", ", 
                    "tkentry(.tableFrame, width='12', textvariable=", varname, ")", sep="")
      }
      #print(make.col.names)
      eval(parse(text=paste("tkgrid(", make.col.names, ", sticky='w')", sep="")), envir=env)
      #print("hello")
      for (i in 1:nrows){   
                make.row <- paste("tklabel(.tableFrame, text='", row.names[i], "')")
                for (j in 1:ncols){
                    varname <- paste(".tab.", i, ".", j, sep="")
                    assign(varname, tclVar(""), envir=env)
                    make.row <- paste(make.row, ", ", "tkentry(.tableFrame, width='5', textvariable=", 
                        varname, ")", sep="")
                   }
                eval(parse(text=paste("tkgrid(", make.row, ", sticky='w')", sep="")), envir=env)
                }
    #  .checkbuttonLabel=tclVar("Check orthogonality")
    #  checkbutton <- tkbutton(.tableFrame, textvariable=.checkbuttonLabel,command=OnCheck, borderwidth=3)
    #  tkgrid(checkbutton, sticky="w")
      tkgrid(get(".tableFrame", envir=env), sticky="w")
    }
    #Check Orthogonaity
   # OnCheck <- function(){
   #   initializeDialog(subdialog,title="Check Orthogonality")
   #   
   #   onOKSub <- function(){
   #     closeDialog()
   #     tkfocus(Commanderwindow())
   #   }
   #   #set up the matrix
    ##  variable <- getSelection(variableBox)
    #  row.names <- eval(parse(text=paste("levels(", .activeDataSet, "$", variable, ")")))
    #  nrows <- length(row.names)
    #  ncols <- nrows - 1
    #  cell <- 0
     # values <- rep(NA,nrows*ncols)
     # nncols <- ncols
     # for (j in 1:ncols){
     ##              if (nncols < ncols) break
     #               for (i in 1:nrows){
     #                   cell <- cell + 1
     #                   varname <- paste(".tab.", i, ".", j, sep="")
     #                   values[cell] <- as.numeric(eval(parse(text=paste("tclvalue(", varname,")", sep=""))))
     #                   #print(as.character(values[cell]))
     #                   if (is.na(values[cell])) {
     #                     #print(ncols)
     #                     nncols <- j-1
      #                    break
     ##                   }
      #                  }
      #}
      #ncols <- nncols
      #contrast.names <- rep("", ncols)
      #for (j in 1:ncols){
      #              varname <- paste(".col.", j, sep="")
      #              contrast.names[j] <- eval(parse(text=paste("tclvalue(", varname,")", sep="")))
      #              assign(paste(".col:", j, sep=""), tkentry(subdialog, width="20", textvariable=eval(parse(text=varname))))
      #              
      #              }
      #
     # OKCancelHelp()
     # tkgrid(outerTableFrame,sticky="w")
      #tkgrid(buttonsFrame, sticky="w")
      ##dialogSuffix(rows=1, columns=1)
    #}
    onOK <- function(){
      #if (GrabFocus()) tkgrab.release(top)
      #  tkdestroy(top)
      variable <- getSelection(variableBox)
      row.names <- eval(parse(text=paste("levels(", .activeDataSet, "$", variable, ")")))
      nrows <- length(row.names)
      ncols <- nrows - 1
      cell <- 0
      values <- rep(NA,nrows*ncols)
      nncols <- ncols
      for (j in 1:ncols){
                   if (nncols < ncols) break
                    for (i in 1:nrows){
                        cell <- cell + 1
                        varname <- paste(".tab.", i, ".", j, sep="")
                        values[cell] <- as.numeric(eval(parse(text=paste("tclvalue(", varname,")", sep=""))))
                        #print(as.character(values[cell]))
                        if (is.na(values[cell])) {
                          #print(ncols)
                          nncols <- j-1
                          break
                        }
                        }
      }
      print(nncols)
      ncols <- nncols
      #values <- values[,1:ncols]
      contrast.names <- rep("", ncols)
                for (j in 1:ncols){
                    varname <- paste(".col.", j, sep="")
                    contrast.names[j] <- eval(parse(text=paste("tclvalue(", varname,")", sep="")))
                    }
      if (GrabFocus()) tkgrab.release(top)
        tkdestroy(top)
      
      command <- paste("matrix(c(", paste(values, collapse=","), "), ", nrows, ", ", ncols,
                    ")", sep="")
                assign(".Contrasts", justDoIt(command), envir=.GlobalEnv)
                logger(paste(".Contrasts <- ", command, sep=""))
      command <- paste("colnames(.Contrasts) <- c(", 
                    paste("': ", contrast.names, "'", sep="", collapse=", "), ")", sep="")
                justDoIt(command)
                logger(command)
      #command <- paste("fit.contrast(",.activeModel,', "',variable,'" , t(.Contrasts))', sep="")
      #          doItAndPrint(command)

      command <- paste("formul <- formula(",.activeModel,'$call)',sep="")
                #doItAndPrint(command)
                justDoIt(command)
                logger(command)
      command <- paste("contrasts(",activeDataSet(),"$", variable,") <-.Contrasts", sep="")
                justDoIt(command)
                logger(command)
      command <- paste("an <-aov(formul,data=",activeDataSet(),")",sep="")
                justDoIt(command)
                logger(command)
      #command <- paste("summary(an,split=list(",variable,"=list('",contrast.names,"'=",1:length(contrast.names),")))",sep="")
      conlist <- paste("'",contrast.names,"'=",1:length(contrast.names),sep="",collapse=",")
      #print(conlist)
      command <- paste("summary(an,split=list(",variable,"=list(",conlist,")))",sep="")
                doItAndPrint(command)
      command <- paste("crossprod(.Contrasts)", sep="")
                doItAndPrint(command)
      command <- paste("a <-crossprod(.Contrasts)", sep="")
                justDoIt(command)
      if (all(a[lower.tri(a)]==TRUE)) {doItAndPrint("print('Orthogonality met')")}
      else {doItAndPrint("print('Orthogonality not met')"); message("Orthogonality not met")}
      remove(formul,envir=.GlobalEnv)
      #remove(.Contrasts,envir=.GlobalEnv)
      #remove(contrast.names,envir=.GlobalEnv)
      remove(an,envir=.GlobalEnv)
    }
    
    OKCancelHelp()
    tkgrid(getFrame(variableBox), sticky="nw")
    tkbind(variableBox$listbox, "<ButtonRelease-1>", FillTable)
    tkgrid(outerTableFrame,sticky="w")
    #tkgrid(tableFrame, sticky="nw")
    tkgrid(buttonsFrame, sticky="w")
    dialogSuffix(rows=4, columns=1) 
}


listDistances <- function(envir=.GlobalEnv, ...) {
    names(which(sapply(ls(envir=envir,all.names=TRUE),
         function(.x) "dist" == (class(eval(parse(text=.x), envir=envir))[1])
         )))
    #objects <- ls(envir=envir, ...)
    #if (length(objects) == 0) NULL
    #else objects[sapply(objects, 
    #    function(.x) "dist" == (class(eval(parse(text=.x), envir=envir))[1]))]
    }



twoWayTableM <- function(){
    if (!checkActiveDataSet()) return()
    if (!checkFactors(2)) return()
    initializeDialog(title="Two-Way Table")
    variablesFrame <- tkframe(top)
    rowBox <- variableListBox(variablesFrame, Factors(), title="Row variable (pick one)")
    columnBox <- variableListBox(variablesFrame, Factors(), title="Column variable (pick one)")
    subsetBox()
    onOK <- function(){
        row <- getSelection(rowBox)
        column <- getSelection(columnBox)
        if (length(row) == 0 || length(column) == 0){
            errorCondition(recall=twoWayTableM, message="You must select two variables.")
            return()
            }
        if (row == column) {
            errorCondition(recall=twoWayTableM, message="Row and column variables are the same.")
            return()
            }        
        percents <- as.character(tclvalue(percentsVariable))
        chisq <- tclvalue(chisqTestVariable)
        expected <- tclvalue(expFreqVariable)
        residuals <-tclvalue(residualsVariable)
    
        fisher <- tclvalue(fisherTestVariable)
        subset <- tclvalue(subsetVariable)
        subset <- if (trim.blanks(subset) == "<all valid cases>") "" 
            else paste(", subset=", subset, sep="")
        if (GrabFocus()) tkgrab.release(top)
        tkdestroy(top)
        command <- paste("xtabs(~", row, "+", column, ", data=", .activeDataSet, 
            subset, ")", sep="")
        logger(paste(".Table <- ", command, sep=""))
        assign(".Table", justDoIt(command), envir=.GlobalEnv)
        doItAndPrint(".Table")
        if (percents == "row") doItAndPrint("rowPercents(.Table) # Row Percentages")
        if (percents == "column") doItAndPrint("colPercents(.Table) # Column Percentages")
        if (chisq == 1) {
            command <- "chisq.test(.Table, correct=FALSE)"
            logger(paste(".Test <- ", command, sep=""))
            assign(".Test", justDoIt(command), envir=.GlobalEnv)
            doItAndPrint(".Test")
            if (expected == 1) doItAndPrint(".Test$expected # Expected Counts")
            warnText <- NULL
            if (0 < (nlt1 <- sum(.Test$expected < 1))) warnText <- paste(nlt1,
                "expected frequencies are less than 1")
            if (0 < (nlt5 <- sum(.Test$expected < 1))) warnText <- paste(warnText, "\n", nlt5,
                " expected frequencies are less than 5", sep="")
            if (!is.null(warnText)) tkmessageBox(message=warnText,
                icon="warning", type="ok")
            if (residuals == 1) doItAndPrint("residuals(.Test)")
            logger("remove(.Test)") 
            remove(.Test, envir=.GlobalEnv) 
            }
        if (fisher == 1) doItAndPrint("fisher.test(.Table)")
        logger("remove(.Table)") 
        remove(.Table, envir=.GlobalEnv)                                                      
        tkfocus(CommanderWindow())
        }
    OKCancelHelp(helpSubject="xtabs")
    radioButtons(name="percents", buttons=c("rowPercents", "columnPercents", "nonePercents"), 
        values=c("row", "column", "none"), initialValue="none", 
        labels=c("Row percentages", "Column percentages", "No percentages"), title="Compute Percentages")
    checkBoxes(frame="testsFrame", boxes=c("chisqTest", "residuals", "expFreq", "fisherTest"), initialValues=c("1", "1", "0", "0"),
        labels=c("Chisquare test of independence", "Residuals", "Print expected frequencies", "Fisher's exact test"))
    tkgrid(getFrame(rowBox), tklabel(variablesFrame, text="    "), getFrame(columnBox), sticky="nw")
    tkgrid(variablesFrame, sticky="w")
    tkgrid(percentsFrame, sticky="w")
    tkgrid(tklabel(top, text="Hypothesis Tests", fg="blue"), sticky="w")
    tkgrid(testsFrame, sticky="w")
    tkgrid(subsetFrame, sticky="w")
    tkgrid(buttonsFrame, sticky="w")
    dialogSuffix(rows=6, columns=1)
    }



tdistdemo <- function(){
local({
    #set all the local variables
    siz <- tclVar(1)
    totalsiz <- tclVar(1)
    meanvar1 <- tclVar(800)
    sdvar1 <- tclVar(20)
    meanvar2 <- tclVar(800)
    sdvar2 <- tclVar(20)
    group1n <-tclVar(8)
    group2n <- tclVar(6)
    tvalues <- dim(1000)
    group1mean <- dim(1000)#just added
    group2mean <- dim(1000)#just added
    #group1mean <- 0
    #group2mean <- 0
    labelText <- tclVar("")
    tvalue <-tclVar("")
    samplemean1 <- tclVar("")
    sm1 <- tclVar("")
    samplemean2 <- tclVar("")
    sm2 <- tclVar("")
    s1 <- dim(1000)
    s2 <- dim(1000)
    specifiedT <- tclVar(0)
    specifiedP <- tclVar(0.05)

    calcT <- tclVar(0)
    calcP <- tclVar(0)
    calcDF <-tclVar(0)
    calcT1 <- tclVar(0)
    calcP1 <- tclVar(0)
    calcDF1 <-tclVar(0)
    tcalc <- tclVar(1)
    radio <- tclVar(1)
    norm1 <- tclVar(1)
    norm2 <- tclVar(1)


tval <- function(...){
	grp1n <- as.numeric(tclObj(group1n))
	grp2n <- as.numeric(tclObj(group2n))
	mn1    <- as.numeric(tclObj(meanvar1))
        sdv1 <-as.numeric(tclObj(sdvar1))
        if(tclvalue(norm1)=="1") m <- rnorm(grp1n,mn1,sdv1)
        else {
          m <- rlnorm(grp1n,log(mn1),sdv1/mn1)
        }
        s1 <<- m
        mn2    <- as.numeric(tclObj(meanvar2))
        sdv2 <-as.numeric(tclObj(sdvar2))
        if(tclvalue(norm2)=="1") f <- rnorm(grp2n,mn2,sdv2)
        else {
          f <- rlnorm(grp2n,log(mn2),sdv2/mn2)
        }
        s2 <<-f
        sm1 <<-mean(m)
        sm2 <<-mean(f)
        group1mean <<- c(group1mean,sm1)  #just added
        group2mean <<- c(group2mean,sm2)  #just added
	tt <- (mean(m)-mean(f))
        varP <- (((length(m)-1)*var(m))+((length(f)-1)*var(f)))/((length(m)-1)+(length(f)-1))
        #print(varP)
        #print(var(m))
        SEm <- varP/length(m)
	SEf <- varP/length(f)
	#SEm <- var(m)/length(m)
	#SEf <- var(f)/length(f)
        #print(t.test(m,f,var.equal=T))
	tval <- tt/sqrt(SEm+SEf)
        
    }	      
progress <- function(n){
  
  tkconfigure(progress.scale, sliderlength=n-3)
  tkcmd("update","idletasks")
  
}
boxp <- function(bb){
  miny <- min(bb$stats)
  maxy <- max(bb$stats)
  miny <- min(c(miny,bb$out))
  maxy <- max(c(maxy,bb$out))
  maxy <- maxy+(maxy*.05)
  miny <- miny-(miny*.05)
  diffy <- maxy-miny
  c.max <-200
  c.min <- 0
  cnvas.height<-c.max-c.min
  
  tkcreate(canvas, "rectangle", 0, 0, 150, 200, width=2,fill="white")
  n <- 20
 for (i in 1:2) {
    b <- bb$stats[,i]
    n <- n + i
    b <- c.max-((maxy-b)*c.max)/(maxy-miny)
    q1 <- b[1]
    q2<- b[2]
    q3 <- b[3]
    q4 <- b[4]
    q5 <- b[5]
    tkcreate(canvas, "rectangle", n, q2, n+30, q4, width=2)
    tkcreate(canvas, "line", n, q3, n+30, q3, width=2)
    tkcreate(canvas, "line", n, q1, n+30, q1, width=2)
    tkcreate(canvas, "line", n+15, q1, n+15, q2, width=2)
    tkcreate(canvas, "line", n, q5, n+30, q5, width=2)
    tkcreate(canvas, "line", n+15, q4, n+15, q5, width=2)
    outliers <-bb$out[bb$group==i]
    if (length(outliers)>0) {
      outliers <- c.max-((maxy-outliers)*c.max)/(maxy-miny) 
      for (j in 1:length(outliers)) tkcreate(canvas,"oval",n+15-2,outliers[j]-2,n+15+2,outliers[j]+2,width=2)
    }
    n <- 75
  }   
}
    
    replot <- function(...) {
	n <- as.numeric(tclObj(siz))
        for (i in 1:n) {
          tvalues[[length(tvalues)+1]] <<- tval()
          progress((i*100)/n)
        }
	eval(substitute(par(mfrow=c(2,1))))
        #eval(substitute(stripchart(zapsmall(tvalues,digits=2),method="stack",xlim=c(-5,5),ylim=c(1,3))))
        #eval(substitute(stripchart(zapsmall(group1mean,digits=2),method="stack")))
        #eval(substitute(hist(group1mean,main="",xlab="",freq=F,breaks=50,xlim=c(min(c(group1mean,group2mean)),max(c(group1mean,group2mean))))))
        eval(substitute(plot(density(group1mean),col="red",xlim=c(min(c(group1mean,group2mean)),max(c(group1mean,group2mean))))))
        if(length(group1mean)>2) {
          eval(substitute(lines(density(group1mean),col="red")))
          eval(substitute(lines(density(group2mean),col="blue")))
        }
        
	#eval(substitute(hist(tvalues,main="",xlab="",freq=F,breaks=50,xlim=c(-5,5))))
        eval(substitute(hist(tvalues,main="",xlab="",freq=F,breaks=50,xlim=c(min(tvalues),max(tvalues)))))
        if(length(tvalues)>2) eval(substitute(lines(density(tvalues))))
	eval(substitute(par(mfrow=c(1,1))))
        tclvalue(labelText) <- as.character(length(tvalues))
        tclvalue(tvalue) <- as.character(round(tvalues[[length(tvalues)]],digits=4))
        tclvalue(samplemean1) <-as.character(round(sm1,digits=2))
        tclvalue(samplemean2) <-as.character(round(sm2,digits=2))

        b <- boxplot(s1,s2,plot=F)
        boxp(b)
        tkconfigure(progress.scale, sliderlength=0)
    }


replot.no.sim <- function(...){
      eval(substitute(par(mfrow=c(2,1))))
	eval(substitute(stripchart(zapsmall(tvalues,digits=2),method="stack",xlim=c(-5,5),ylim=c(1,3))))
	eval(substitute(hist(tvalues,main="",xlab="",freq=F,breaks=50,xlim=c(-5,5))))
	eval(substitute(par(mfrow=c(1,1))))
        #tclvalue(labelText) <- as.character(length(tvalues))
        #tclvalue(tvalue) <- as.character(round(tvalues[[length(tvalues)]],digits=4))
        #tclvalue(samplemean1) <-as.character(round(sm1,digits=2))
        #tclvalue(samplemean2) <-as.character(round(sm2,digits=2))
    }

resimulate <- function(...){
     tvalues <- 0
     group1mean <-0 #just added
     group1mean <- 0#just added
     group2mean <<- dim(1000)#just added
     group2mean <<- dim(1000)#just added
     #print(tvalues)
     tvalues <<- dim(1000)
     replot()
}
    overlayt <- function(...) {
        grp1n <- as.numeric(tclObj(group1n))
	grp2n <- as.numeric(tclObj(group2n))
        eval(substitute(par(mfrow=c(2,1))))
        #min <- round(qt(0.0005,df=(grp1n-1)+(grp2n-1)),3)
        #max <- round(qt(0.9995,df=(grp1n-1)+(grp2n-1)),3)
        min <--5
        max <- 5
        .x <- seq(min,max, length=100)
	eval(substitute(lines(.x,dt(.x,df=(grp1n-1)+(grp2n-1)),col="blue")))
        eval(substitute(par(mfrow=c(1,1))))
    }

    overlayPoly <- function(...){
        replot.no.sim()
        grp1n <- as.numeric(tclObj(group1n))
	grp2n <- as.numeric(tclObj(group2n))
        eval(substitute(par(mfrow=c(2,1))))
        #min <- round(qt(0.0005,df=(grp1n-1)+(grp2n-1)),3)
        #max <- round(qt(0.9995,df=(grp1n-1)+(grp2n-1)),3)
        min <- -5
        max <- 5
        .x <- seq(min,max, length=100)
        
        if(tclvalue(tcalc)=="1")  {
          t <- as.numeric(tclObj(specifiedT)) #if t critical t givern
          tclvalue(calcT1) <-as.character(round(t,digits=3))
          tclvalue(calcDF1) <-as.character(round((grp1n-1)+(grp2n-1),digits=3))
          if (tclvalue(radio)=="1") {  #one tailed test
            tclvalue(calcP1) <- as.character(length(tvalues[tvalues>t])/length(tvalues))
          }
          else{
            tclvalue(calcP1) <- as.character((length(tvalues[tvalues>t])/length(tvalues))+(length(tvalues[tvalues< -t])/length(tvalues)))
          }
        }
        else {        #if p value given
          p <- as.numeric(tclObj(specifiedP))
          tclvalue(calcP1) <- as.character(round(p,digits=3))
          tclvalue(calcDF1) <-as.character(round((grp1n-1)+(grp2n-1),digits=3))
          if (tclvalue(radio)=="1") {  #one tailed test
            t <- qt(p,df=(grp1n-1)+(grp2n-1),lower=F)
            tclvalue(calcT1) <-as.character(round(sort(tvalues)[length(tvalues)-length(tvalues)*p],digits=3))
          }
          else {  #two-tailed test
            t <- qt((p/2),df=(grp1n-1)+(grp2n-1),lower=F)
            tclvalue(calcT1) <-as.character(round(sort(tvalues)[length(tvalues)-(length(tvalues)*(p/2))],digits=3))
          }
        }
        
        #.xx <- c(max(.x),.x[.x>t],t,t)
        #.yy <- c(0,dt(.x[.x>t],df=(grp1n-1)+(grp2n-1)),dt(t,df=(grp1n-1)+(grp2n-1)),0)
        .xx <- c(t,.x[.x>t],max(.x),t)
        .yy <- c(dt(t,df=(grp1n-1)+(grp2n-1)),dt(.x[.x>t],df=(grp1n-1)+(grp2n-1)),0,0)
        eval(substitute(polygon(.xx,.yy,col="red")))
        if(tclvalue(radio)=="1"){    #one tail test
          pp <- pt(t,df=(grp1n-1)+(grp2n-1),lower=F)
          tclvalue(calcP) <-as.character(round(pp,digits=3))
          tclvalue(calcT) <-as.character(round(t,digits=3))
          tclvalue(calcDF) <-as.character(round((grp1n-1)+(grp2n-1),digits=3))

          #tclvalue(calcP1) <- as.character(length(tvalues[tvalues>t])/length(tvalues))
          
          #tclvalue(calcDF1) <-as.character(round((grp1n-1)+(grp2n-1),digits=3))
        }
        else {  # two-tailed test
          t <- t*-1
          .xx <- c(min(.x),.x[.x<t],t,t)
          .yy <- c(0,dt(.x[.x<t],df=(grp1n-1)+(grp2n-1)),dt(t,df=(grp1n-1)+(grp2n-1)),0)
          eval(substitute(polygon(.xx,.yy,col="red")))
          t<-t*-1
          pp <- pt(t,df=(grp1n-1)+(grp2n-1),lower=F)*2
          tclvalue(calcP) <-as.character(round(pp,digits=3))
          tclvalue(calcT) <-as.character(round(t,digits=3))
          tclvalue(calcDF) <-as.character(round((grp1n-1)+(grp2n-1),digits=3))

          #tclvalue(calcP1) <- as.character((length(tvalues[tvalues>t])/length(tvalues))+(length(tvalues[tvalues< -t])/length(tvalues)))
          #tclvalue(calcT1) <-as.character(round(t,digits=3))
          #tclvalue(calcDF1) <-as.character(round((grp1n-1)+(grp2n-1),digits=3))
        }
        
        eval(substitute(par(mfrow=c(1,1))))
        overlayt()
    }
    changeTcalc <- function(...){
       
    } 


    reset <- function(...) {
        tvalues <<- dim(1000)
        group1mean <<- dim(1000)#just added
        group2mean <<- dim(1000)#just added
        #siz <<- tclVar(1)
        #totalsiz <<- tclVar(1)
        #meanvar1 <<- tclVar(800)
        #sdvar1 <<- tclVar(20)
        #meanvar2 <<- tclVar(800)
        #sdvar2 <<- tclVar(20)
        #group1n <<-tclVar(8)
        #group2n <<- tclVar(6)
        #labelText <<- tclVar("")
        #tvalue <<-tclVar("")
        #samplemean1 <<- tclVar("")
        #sm1 <<- tclVar("")
        #samplemean2 <<- tclVar("")
        #sm2 <<- tclVar("")
        #specifiedT <<- tclVar(0)
        #specifiedP <<- tclVar(0.05)

        #calcT <- tclVar(1)
        #calcP <- tclVar(0)
        #calcDF <-tclVar(0)
        #tcalc <- tclVar(1)
        #radio <- tclVar(1)
        tclvalue(labelText) <- as.character(0)
        tclvalue(tvalue) <- as.character(0)
        tclvalue(samplemean1) <- as.character(0)
        tclvalue(samplemean2) <- as.character(0)
        tclvalue(calcT) <- as.character(0)
        tclvalue(calcP) <- as.character(0)
        tclvalue(calcDF) <- as.character(0)
        tclvalue(meanvar1) <- as.numeric(800)
        tclvalue(meanvar2) <- as.numeric(800)
        tclvalue(sdvar1) <- as.numeric(20)
        tclvalue(sdvar2) <- as.numeric(20)
        tclvalue(group1n) <- as.numeric(8)
        tclvalue(group2n) <- as.numeric(6)
        replot()
        #tclvalue(mean1.entry) <- tclvalue(meanvar1)
        
    }

    regen <- function(...) {
        #if (tclvalue(dist)=="1") y<<-rnorm(as.numeric(tclObj(size)))
        #else y<<-rexp(as.numeric(tclObj(size)))
        #xlim <<- range(y) + c(-2,2)
        replot()
    }





    
    base <- tktoplevel()
    tkwm.title(base, "t distribution demo")


    #population 1 input
    frame1 <- tkframe(base, relief="groove", borderwidth=2,width=100)
    tkgrid(tklabel(frame1, text="Population 1",fg="blue"),sticky="nsew")
    mean1.entry <- tkentry(frame1,text=meanvar1,width=5)
    tkconfigure(mean1.entry,textvariable=meanvar1)
    tkbind(mean1.entry, "<Return>", replot)
    tkgrid(tklabel(frame1, text="Mean"),mean1.entry,sticky="e")
    sd1.entry <- tkentry(frame1,textvariable=sdvar1,width=5)
    tkbind(sd1.entry, "<Return>", replot)
    tkgrid(tklabel(frame1, text="SD"),sd1.entry,sticky="e")
    norm.btn1 <- tkcheckbutton(frame1,text="Normal distribution",variable=norm1)
    tkgrid(norm.btn1,sticky="")
    tkgrid(tklabel(frame1,text=""))
    g1n.entry <- tkentry(frame1,textvariable=group1n, width=5)
    tkbind(g1n.entry, "<Return>", replot)	   
    tkgrid(tklabel(frame1, text="Sample size"),g1n.entry,sticky="e")
    
    #population 2 input
    frame2 <- tkframe(base, relief="groove", borderwidth=2,width=200)
    tkgrid(tklabel(frame2, text="Population 2",fg="blue"),sticky="nsew")
mean2.entry <- tkentry(frame2,textvariable=meanvar2,width=5)
    tkbind(mean2.entry, "<Return>", replot)
    tkgrid(tklabel(frame2, text="Mean"),mean2.entry,tklabel(frame2, text="",width=7),sticky="e")
    sd2.entry <- tkentry(frame2,textvariable=sdvar2,width=5)
    tkbind(sd2.entry, "<Return>", replot)	   
    tkgrid(tklabel(frame2, text="SD"),sd2.entry,sticky="e")
    norm.btn2 <- tkcheckbutton(frame2,text="Normal distribution",variable=norm2)
    tkgrid(norm.btn2,sticky="e")
    tkgrid(tklabel(frame2,text=""))
    g2n.entry <- tkentry(frame2,textvariable=group2n, width=5)
    tkbind(g2n.entry, "<Return>", replot)	   
    tkgrid(tklabel(frame2, text="Sample size"),g2n.entry)


    
    #Simulations input
    frame3 <- tkframe(base, relief="groove", borderwidth=2,width=200)
    tkgrid(tklabel(frame3, text="Simulations",fg="blue"))
sz.entry <- tkentry(frame3,textvariable=siz,width=5)
    tkbind(sz.entry, "<Return>", replot)
    tkgrid(tklabel(frame3, text="Number of simulations"),sz.entry)
    collectbtn <- tkbutton(frame3,text="Collect sample(s)",command=replot)
    tkgrid(collectbtn)
    recollectbtn <- tkbutton(frame3,text="Re collect sample(s)",command=resimulate)
    tkgrid(recollectbtn)
    progress.scale <- tkscale(frame3,from=0,to=100,borderwidth=0,orient="horiz",showvalue=0,sliderlength=0,sliderrelief="flat",background="blue")
    tkgrid(progress.scale,rowspan=2)
    
    #Simulation results output
    frame4 <- tkframe(base, relief="groove", borderwidth=2,width=200)
    tkgrid(tklabel(frame4, text="Results",fg="blue"))
totalsz.text <- tklabel(frame4,text=tclvalue(labelText))
    tkconfigure(totalsz.text,textvariable=labelText)
    tkgrid(tklabel(frame4,text="Total simulations:"),totalsz.text)
    tval.text <- tklabel(frame4,text=tclvalue(tvalue))
    tkconfigure(tval.text,textvariable=tvalue)
    tkgrid(tklabel(frame4,text="Last t value:"),tval.text)
    samplemean1.text <- tklabel(frame4,text=tclvalue(samplemean1))
    tkconfigure(samplemean1.text,textvariable=samplemean1)
    tkgrid(tklabel(frame4,text="Last sample 1 mean:"),samplemean1.text)
    samplemean2.text <- tklabel(frame4,text=tclvalue(samplemean2))
    tkconfigure(samplemean2.text,textvariable=samplemean2)
    tkgrid(tklabel(frame4,text="Last sample 2 mean:"),samplemean2.text)
    
    #boxplot
    bargraph.frm <- tkframe(base, relief="groove", borderwidth=2)
    canvas <- tkcanvas(bargraph.frm, relief="raised", width=150, height=200)
    tkgrid(canvas,rowspan=2)

    #t distribution overlays
    frame5 <- tkframe(base, relief="groove", borderwidth=2)
    tkgrid(tklabel(frame5, text="t distribution",fg="blue"))
addtdistbtn <- tkbutton(frame5,text="Overlay t distribution",command=overlayt)
    tkgrid(addtdistbtn,tklabel(frame5, text="",width=4),columnspan=2)
    tkgrid(tkradiobutton(frame5, command=overlayPoly, text="One-tailed",
                         value=1, variable=radio))
    tkgrid(tkradiobutton(frame5, command=overlayPoly, text="Two-tailed",
                         value=2, variable=radio))
    tkgrid(tklabel(frame5,text=""))
    specT.entry <- tkentry(frame5,textvariable=specifiedT,width=5)
    tkbind(specT.entry, "<Return>", overlayPoly)
    tkgrid(tkradiobutton(frame5, command=overlayPoly, text="t > :",
                         value=1,variable=tcalc),specT.entry,sticky="e")
    
    specP.entry <- tkentry(frame5,textvariable=specifiedP,width=5)
    tkbind(specP.entry, "<Return>", overlayPoly)
    tkgrid(tkradiobutton(frame5, command=overlayPoly, text="p < :",
                         value=2, variable=tcalc),specP.entry,sticky="e")
    
    #t distribution results
    frame6 <- tkframe(base, relief="groove", borderwidth=2)
    tkgrid(tklabel(frame6, text="Results based on t distribution",fg="blue"))
    calcT.text <- tklabel(frame6,text=tclvalue(calcT))
    tkconfigure(calcT.text,textvariable=calcT)
    tkgrid(tklabel(frame6,text="Critical t value:"),calcT.text)
    calcP.text <- tklabel(frame6,text=tclvalue(calcP))
    tkconfigure(calcP.text,textvariable=calcP)
    tkgrid(tklabel(frame6,text="Probability:"),calcP.text)
    calcDF.text <- tklabel(frame6,text=tclvalue(calcDF))
    tkconfigure(calcDF.text,textvariable=calcDF)
    tkgrid(tklabel(frame6,text="Degrees of Freedom:"),calcDF.text)
    
    tkgrid(tklabel(frame6, text="Results based on sampling distribution",fg="blue"))
    calcT1.text <- tklabel(frame6,text=tclvalue(calcT1))
    tkconfigure(calcT1.text,textvariable=calcT1)
    tkgrid(tklabel(frame6,text="Critical t value:"),calcT1.text)
    calcP1.text <- tklabel(frame6,text=tclvalue(calcP1))
    tkconfigure(calcP1.text,textvariable=calcP1)
    tkgrid(tklabel(frame6,text="Probability:"),calcP1.text)
    calcDF1.text <- tklabel(frame6,text=tclvalue(calcDF1))
    tkconfigure(calcDF1.text,textvariable=calcDF1)
    tkgrid(tklabel(frame6,text="Degrees of Freedom:"),calcDF1.text)
    
    #put the buttons at the end
    bs.frm <- tkframe(base)
    q.but <- tkbutton(bs.frm,text="Quit",
                      command=function()tkdestroy(base))
    reset.but <- tkbutton(bs.frm,text="Reset",
                      command=reset)
    tkgrid(q.but, reset.but)
    
    #draw all the grids at the end
    tkgrid(frame1,row=0,column=0,sticky="news")
    tkgrid(frame2,row=0,column=1,sticky="news")
    tkgrid(frame3,row=1,column=0,sticky="news")
    tkgrid(frame4,row=2,column=0,sticky="news")
    tkgrid(bargraph.frm,row=1,column=1,rowspan=2,sticky="news")
    tkgrid(frame5,row=3,column=0,sticky="news")
    tkgrid(frame6,row=3,column=1,sticky="news")
    tkgrid(bs.frm,row=4,column=0,columnspan=2,sticky="news")
    cat("******************************************************\n",
        "The source for this demo of the principals of hypothesis;\n",
        "testing from a inferential statistics perspective\n",
        "\n******************************************************\n")
    
})
}



anovaTableM <- function(){
    if (!checkActiveModel()) return()
    .models <- union(listLinearModels(), listGeneralizedLinearModels())
    if (length(.models) == 0){
        tkmessageBox(message="There are no models from which to choose.", 
                icon="error", type="ok")
        tkfocus(CommanderWindow())
        return()
        }
    initializeDialog(title="Anova table")
    modelsBox <- variableListBox(top, .models, title="Models (pick one)")
    errorsBox <- variableListBox(top, .models, title="Error terms model (pick one)")
    
    env <- environment()
    tableFrame <- tkframe(top)
    to.split <- tclVar(0)
    onSplit <- function(){
      model <- getSelection(modelsBox)
      if (length(model) == 0) {
        errorCondition(recall=anovaTableM, message=paste("No linear model selected", sep=""))
        #anovaTableM()
        return()
      }
      if (tclvalue(to.split)=="0") {
        for (i in 1:nfactors) {
          ncols <- length(eval(parse(text=paste("levels(", ActiveDataSet(), "$", row.names[i], ")"))))-1
          for (j in 1:ncols) {
            varname <- paste(".tab.", i, ".", j, sep="")
            VarEntry <- eval(parse(text=varname))
            #print(VarEntry)
            tclvalue(VarEntry) <- paste("")
          }
        }
      }
      else {
        nfactors=length(eval(parse(text=paste("names(",ActiveModel(),"$contrasts)",sep=""))))
        row.names <- eval(parse(text=paste("names(",ActiveModel(),"$contrasts)",sep="")))
        for (i in 1:nfactors) {  
          make.row <- paste("tklabel(tableFrame, text='", row.names[i], "')")
          ncols <- length(eval(parse(text=paste("levels(", ActiveDataSet(), "$", row.names[i], ")"))))-1
          split <- ""
          for (j in 1:ncols) {
            varname <- paste(".tab.", i,".",j, sep="")
            if (eval(parse(text=paste("is.null(colnames(contrasts(",ActiveDataSet(),"$", row.names[i],"))[",j,"])",sep="")))) {
              contrast.name<-""}
            else {
              contrast.name <- eval(parse(text=paste("colnames(contrasts(",ActiveDataSet(),"$", row.names[i],"))[",j,"]",sep="")))
          #if (split=="") split <- paste(row.names[i], "=list(A=1)",sep="")
          #else split <- paste(split,",",row.names[i], "=list(A=1)",sep="")
            }
            VarEntry <- eval(parse(text=varname))
            tclvalue(VarEntry) <- paste(contrast.name)
            #assign(varname, tclVar(contrast.name), envir=env)
            #make.row <- paste(make.row, ", ", "tkentry(tableFrame, width='5', textvariable=", 
            #            varname, ")", sep="")
          }
          #eval(parse(text=paste("tkgrid(", make.row, ", sticky='w')", sep="")), envir=env)
        }
      }
        
    }                      
    split.btn <- tkcheckbutton(tableFrame,text="Split ANOVA table",variable=to.split, command=onSplit)
    tkgrid(split.btn,sticky="e")                      
    

    nfactors=length(eval(parse(text=paste("names(",ActiveModel(),"$contrasts)",sep=""))))
    row.names <- eval(parse(text=paste("names(",ActiveModel(),"$contrasts)",sep=""))) #names(ActiveModel()$contrasts)
    for (i in 1:nfactors) {  
      make.row <- paste("tklabel(tableFrame, text='", row.names[i], "')")
      ncols <- length(eval(parse(text=paste("levels(", ActiveDataSet(), "$", row.names[i], ")"))))-1
      split <- ""
      for (j in 1:ncols) {
        varname <- paste(".tab.", i,".",j, sep="")
        if (eval(parse(text=paste("is.null(colnames(contrasts(",ActiveDataSet(),"$", row.names[i],"))[",j,"])",sep="")))) {
          contrast.name<-""}
        else {
          contrast.name <- eval(parse(text=paste("colnames(contrasts(",ActiveDataSet(),"$", row.names[i],"))[",j,"]",sep="")))
        }
        #assign(varname, tclVar(contrast.name), envir=env)
        assign(varname, tclVar(""), envir=env)
        make.row <- paste(make.row, ", ", "tkentry(tableFrame, width='5', textvariable=", 
                        varname, ")", sep="")
      }
      eval(parse(text=paste("tkgrid(", make.row, ", sticky='w')", sep="")), envir=env)
    }
   #tkgrid(tableFrame, sticky="w")
    onOK <- function(){
          model <- getSelection(modelsBox)
          if (length(model) == 0) {
            if (GrabFocus()) tkgrab.release(top)
            tkdestroy(top)
            tkfocus(CommanderWindow())
            return()
            }

         if(tclvalue(to.split)=="1") {        
          split <- paste(",split=list(")
          split1 <- ""
          cell<-0
          fact <- list()
          for (i in 1:nfactors) {
            ncols <- length(eval(parse(text=paste("levels(", ActiveDataSet(), "$", row.names[i], ")"))))-1
             #ifelse(substr(split, nchar(split),nchar(split)) !="(",
             #       split <- paste(split, ",",row.names[i], "=list(", sep=""),
             #       split <- paste(split,row.names[i], "=list(", sep="")
             #       )
             ifelse(split1!="",
                    split1 <- paste(split1, ",",row.names[i], "=list(", sep=""),
                    split1 <- paste(split1,row.names[i], "=list(", sep="")
                    )
            fact[i] <- ""
            for (j in 1:ncols) {
              cell <- cell + 1
              varname <- paste(".tab.", i, ".", j, sep="")
              values<- (eval(parse(text=paste("tclvalue(", varname,")", sep=""))))
              VarEntry <- eval(parse(text=varname))
              if (tclvalue(VarEntry)!="") {
                ifelse(substr(split1, nchar(split1),nchar(split1)) !="(",
                     #split <- paste(split, ",","A","=",j,sep=""), #tclvalue(varname[i][j])
                       split1 <- paste(split1, ",'",values,"'=",j,sep=""), #values[cell],
                       split1 <- paste(split1, "'",values,"'=",j,sep="")
                       )
                fact[[i]] <- c(fact[[i]],paste(values))
              }
            }
            split1 <- paste(split1,")", sep="")
            #print(split1)
            if (all(fact[[i]]=="")) {split1<-""}
          }
          split <- paste(split,split1,")", sep="")
          #print(split)
        }
        else split <-""
          #split <- paste(split,")", sep="")
          error <- getSelection(errorsBox)
          if (!length(error==0)) {
            justDoIt(paste(".aov <- Anova(", model, ")", sep=""))
            doItAndPrint(paste("summary(aov(", model,")", split, ")",sep=""))
          }
          else {
            doItAndPrint(paste(".aov <- Anova(", model, ",error=",error,")", sep=""))
          }
            #justDoIt(".aov[5] <-.aov[4]")
            #justDoIt(".aov[4] <-.aov[3]")
            #justDoIt(".aov[3] <-.aov[1]/.aov[2]")
            #justDoIt("names(.aov) <- c('Sums Sq', 'Df', 'Mean Sq', 'F value', 'Pr(>F)')")
           # doItAndPrint(".aov")            
        if (GrabFocus()) tkgrab.release(top)
        tkdestroy(top)
        tkfocus(CommanderWindow())
              
    }
    OKCancelHelp()
    nameFrame <- tkframe(top)
    tkgrid(nameFrame, sticky="w", columnspan="2")
    tkgrid(getFrame(modelsBox), columnspan="2", sticky="w")
    tkgrid(getFrame(errorsBox), columnspan="2", sticky="w")
    tkgrid(tableFrame, sticky="w")
    #tkgrid(tableFrame, sticky="w", columnspan="2")                      
    tkgrid(buttonsFrame, columnspan=3, sticky="w")
    dialogSuffix(rows=3, columns=3)
}

setContrastsM <- function(){
#    if (!checkActiveDataSet()) return()
#    if (!checkFactors()) return()
    initializeDialog(title="Set Contrasts for Factor")
    variableBox <- variableListBox(top, Factors(), title="Factor (pick one)")
    radioButtons(name="contrasts", buttons=c("treatment", "sum", "helmert", "poly", "specify"), 
        values=c("contr.Treatment", "contr.Sum", "contr.helmert", "contr.poly", "specify"),
        labels=c("Treatment (dummy) contrasts", "Sum (deviation) contrasts", "Helmert contrasts",
            "Polynomial contrasts", "Other (specify)"), title="Contrasts")
    onOK <- function(){
        variable <- getSelection(variableBox)
        closeDialog()
        if (length(variable) == 0) {
            errorCondition(recall=setContrasts, message="You must select a variable.")
            return()
            }
        contrasts <- tclvalue(contrastsVariable)
        if (contrasts != "specify"){
            command <- paste("contrasts(", ActiveDataSet(), "$", variable, ') <- "', contrasts, '"', sep="")
            justDoIt(command)
            logger(command)
            activeDataSet(ActiveDataSet())
            tkfocus(CommanderWindow())
            }
        else{
            initializeDialog(subdialog, title="Specify Contrasts")
            tkgrid(tklabel(subdialog, text="Enter Contrast Coefficients", fg="blue"), sticky="w")
            env <- environment()
            tableFrame <- tkframe(subdialog)
            row.names <- eval(parse(text=paste("levels(", ActiveDataSet(), "$", variable, ")")))
            row.names <- substring(paste(abbreviate(row.names, 12), "            "), 1, 12)
            nrows <- length(row.names)
            ncols <- nrows - 1
            make.col.names <- "tklabel(tableFrame, text='Contrast Name:')"
            for (j in 1:ncols) {
                varname <- paste(".col.", j, sep="")
                assign(varname, tclVar(paste(".", j, sep="")), envir=env)
                make.col.names <- paste(make.col.names, ", ", 
                    "tkentry(tableFrame, width='12', textvariable=", varname, ")", sep="")
                }
            eval(parse(text=paste("tkgrid(", make.col.names, ", sticky='w')", sep="")), envir=env)
            for (i in 1:nrows){   
                make.row <- paste("tklabel(tableFrame, text='", row.names[i], "')")
                for (j in 1:ncols){
                    varname <- paste(".tab.", i, ".", j, sep="")
                    assign(varname, tclVar(""), envir=env)
                    make.row <- paste(make.row, ", ", "tkentry(tableFrame, width='5', textvariable=", 
                        varname, ")", sep="")
                    }
                eval(parse(text=paste("tkgrid(", make.row, ", sticky='w')", sep="")), envir=env)
                }
            tkgrid(tableFrame, sticky="w")
            onOKsub <- function(){
                closeDialog(subdialog)
                cell <- 0
                values <- rep(NA, nrows*ncols)
                for (j in 1:ncols){
                    for (i in 1:nrows){
                        cell <- cell + 1
                        varname <- paste(".tab.", i, ".", j, sep="")
                        values[cell] <- as.numeric(eval(parse(text=paste("tclvalue(", varname,")", sep=""))))
                        }
                    }
                values <- na.omit(values)
                ncols1=length(values)/nrows
                #if (length(values) != nrows*ncols){
                #    errorCondition(subdialog, recall=setContrasts,
                #        message=paste("Number of valid entries in contrast matrix(", length(values), ")\n",
                #            "not equal to number of levels (", nrows,") * number of contrasts (", ncols,").", 
                #            sep=""))
                #    return()
                #    }
              #  if (qr(matrix(values, nrows, ncols))$rank < ncols) {
              #      errorCondition(subdialog, recall=setContrasts, message="Contrast matrix is not of full column rank")
              #      return()
              #      }  
                contrast.names <- rep("", ncols)
                for (j in 1:ncols){
                    varname <- paste(".col.", j, sep="")
                    contrast.names[j] <- eval(parse(text=paste("tclvalue(", varname,")", sep="")))
                    }
                if (length(unique(contrast.names)) < ncols) {
                    errorCondition(subdialog, recall=setContrasts, message="Contrast names must be unique") 
                    return()
                    }                    
                #command <- paste("matrix(c(", paste(values, collapse=","), "), ", nrows, ", ", ncols,
                #    ")", sep="")
                command <- paste("matrix(c(", paste(values, collapse=","), "), ", nrows, ", ", ncols1,
                    ")", sep="")
                assign(".Contrasts", justDoIt(command), envir=.GlobalEnv)
                orth <- crossprod(.Contrasts)
                if (all(orth[lower.tri(orth)]==0)==FALSE){
                  errorCondition(subdialog, recall=setContrasts, message="Contrasts not orthogonal")
                  return()
                   }
                logger(paste(".Contrasts <- ", command, sep=""))
                command <- paste("contrasts(", ActiveDataSet(), "$", variable, ") <- .Contrasts", sep="")
                justDoIt(command)
                logger(command)
                command <- paste("colnames(contrasts(", ActiveDataSet(), "$", variable, ")) <- c(", 
                    paste("'", contrast.names, "'", sep="", collapse=", "), ")", sep="")
                justDoIt(command)
                logger(command)
                command <- paste("contrasts(",ActiveDataSet(), "$", variable, ")", sep="")
                doItAndPrint(command)
                #logger(command)
                command <- paste("round(crossprod(contrasts(",ActiveDataSet(), "$", variable, ")), digits=2)", sep="")
                doItAndPrint(command)
                #logger(command)
                justDoIt("remove(.Contrasts, envir=.GlobalEnv)")   
                logger("remove(.Contrasts)") 
                activeDataSet(ActiveDataSet(), flushModel=FALSE)
                tkfocus(CommanderWindow())
                }
            subOKCancelHelp(helpSubject="contrasts")
            tkgrid(tableFrame, sticky="w")
            tkgrid(tklabel(subdialog, text=""))
            tkgrid(subButtonsFrame, sticky="w")
            dialogSuffix(subdialog, rows=5, columns=1, focus=subdialog)  
            }
        }
    OKCancelHelp(helpSubject="contrasts")
    tkgrid(getFrame(variableBox), sticky="nw")
    tkgrid(contrastsFrame, sticky="w")
    tkgrid(buttonsFrame, sticky="w")
    dialogSuffix(rows=4, columns=1) 
    }
distanceP <- function(n=0) length(listDistances()) >n
